/*
Copyright (C) 2008-2009  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <rc_shiftreg.h>
#include <rc_ppm-encoder.h>
#include <rc_adc.h>
#include <rc_control.h>
#include <spi_sw.h>
#include <sd.h>
#include <glcd.h>
#include <sound.h>
#include <misc.h>


void delay_ms (uint32_t time) {
	uint32_t counter = time * 5400;
	while (counter > 0) {
		counter--;
		asm volatile ("nop");
	}
}

uint16_t add_csum (uint8_t *buffer, uint16_t len) {
	uint16_t n;
	uint8_t csum = 255;
	for (n = 0; n < len; n++) {
		csum ^= buffer[n];
	}
	buffer[len++] = csum;
	return len;
}

char Rc_REncoderDecode_menu_byte (uint8_t byte, uint8_t bit_a, uint8_t bit_b, uint8_t bit_sw) {
	static int8_t last = 0;
	int8_t new = 0;
	int8_t diff;
	int8_t delta;
	if(! (byte & (1<<bit_a)) ) {
		new = 3;
	}
	if(! (byte & (1<<bit_b)) ) {
		new ^= 1;
	}
	if(! (byte & (1<<bit_sw)) ) {
		return CMD_ENTER;
	}
	diff = last - new;
	if(diff & 1) {
		last = new;
		delta = (diff & 2) - 1;
		if (delta > 0) {
			return CMD_DOWN;
		} else if (delta < 0) {
			return CMD_UP;
		}
	}
	return 0;
}

char *int2str_new (char *str, int16_t value, uint8_t mode, uint8_t digit, uint8_t close_str) {
	uint8_t n = 0;
	uint8_t val = 0;
	if (value < 0) {
		value *= -1;
		str[n++] = '-';
	}
	if (digit == 4) {
		str[n++] = '.';
	}
	val = (value / 1000);
	if (mode >= 3 || val > 0) {
		str[n++] = val + 0x30;
		mode = 1;
	}
	if (digit == 3) {
		str[n++] = '.';
	}
	val = ((value % 1000) / 100);
	if (mode >= 2 || val > 0) {
		str[n++] = val + 0x30;
		mode = 1;
	}
	if (digit == 2) {
		str[n++] = '.';
	}
	val = (((value % 1000) % 100) / 10);
	if (mode >= 1 || val > 0) {
		str[n++] = val + 0x30;
		mode = 1;
	}
	if (digit == 1) {
		str[n++] = '.';
	}
	str[n++] = (((value % 1000) % 100) % 10) + 0x30;
	if (close_str == 1) str[n++] = 0;
	return str;
}

char *int2str (char *str, int value, char mode) {
	if (value >= 10000) {
		str[0] = (value / 10000) + 48;
		value = value - (value / 10000 * 10000);
		str[1] = (value / 1000) + 48;
		value = value - (value / 1000 * 1000);
		str[2] = (value / 100) + 48;
		value = value - (value / 100 * 100);
		str[3] = (value / 10) + 48;
		value = value - (value / 10 * 10);
		str[4] = value + 48;
		if (mode == 0) str[5] = 0;
	} else if (value >= 1000) {
		str[0] = (value / 1000) + 48;
		value = value - (value / 1000 * 1000);
		str[1] = (value / 100) + 48;
		value = value - (value / 100 * 100);
		str[2] = (value / 10) + 48;
		value = value - (value / 10 * 10);
		str[3] = value + 48;
		if (mode == 0) str[4] = 0;
	} else if (value >= 100) {
		str[0] = (value / 100) + 48;
		value = value - (value / 100 * 100);
		str[1] = (value / 10) + 48;
		value = value - (value / 10 * 10);
		str[2] = value + 48;
		if (mode == 0) str[3] = 0;
	} else if (value >= 10) {
		str[0] = (value / 10) + 48;
		value = value - (value / 10 * 10);
		str[1] = value + 48;
		if (mode == 0) str[2] = 0;
	} else if (value >= 0) {
		str[0] = value + 48;
		if (mode == 0) str[1] = 0;
	}
	return str;
}

