
#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <rc_adc.h>
#include <glcd.h>
#include <spi_sw.h>
#include <dogm128.h>

#ifdef SUPPORT_LCD
#ifdef SUPPORT_DOGM128

static void DOGM128_Send (uint8_t data, uint8_t cd) {
	if(cd == LCD_DATA) {
		GLCD_DS_HI;
	} else {
		GLCD_DS_LO;
	}
	GLCD_SEL_LO;
	GLCD_Write(data);
	GLCD_SEL_HI;
}

void DOGM128_Init (void) {
	GLCD_RST_LO;
	delay_ms(50);
	GLCD_RST_HI;
	delay_ms(50);

	DOGM128_Send(DOGM128_START_ADDR, LCD_CMD);
#ifdef LCD_FLIP
	DOGM128_Send(DOGM128_ADC_REVERSE + 1, LCD_CMD);
	DOGM128_Send(DOGM128_NORMAL_COM0_63, LCD_CMD);
#else
	DOGM128_Send(DOGM128_ADC_REVERSE, LCD_CMD);
	DOGM128_Send(DOGM128_NORMAL_COM63_0, LCD_CMD);
#endif
	DOGM128_Send(DOGM128_DISPLAY_NORMAL, LCD_CMD);
	DOGM128_Send(DOGM128_BIAS1_9, LCD_CMD);
#ifdef DOGM128_BOOST
	DOGM128_Send(DOGM128_BOOSTER_REG_FOL, LCD_CMD);
//	DOGM128_Send(DOGM128_BOOSTER_4X, LCD_CMD);
#else
	DOGM128_Send(DOGM128_BOOSTER_OFF, LCD_CMD);
#endif

#ifdef LCD_INVERT
	DOGM128_Send(DOGM128_ADC_INVERT + 1, LCD_CMD);
#endif
	DOGM128_Send(DOGM128_CONTRAST_SET1, LCD_CMD);
	DOGM128_Send(DOGM128_CONTRAST_SET2, LCD_CMD);
	DOGM128_Send(DOGM128_CONTRAST_SET3, LCD_CMD);
	DOGM128_Send(DOGM128_INDICATOR_NO1, LCD_CMD);
	DOGM128_Send(DOGM128_INDICATOR_NO2, LCD_CMD);
	DOGM128_Send(DOGM128_DISPLAY_ON, LCD_CMD);
}

void DOGM128_Update (void) {
	uint16_t n;
	uint16_t n2;
	uint16_t n3;
	SPI_Fast(); // force Fast-SPI
	n2 = 0;
	for (n3 = 0; n3 < LCD_H / 8; n3++) {
		DOGM128_Send(0xB0 + n3, LCD_CMD); // Page
		DOGM128_Send(0x10, LCD_CMD); // Column MSB
		DOGM128_Send(0x00, LCD_CMD); // Column LSB
		GLCD_DS_HI;
		GLCD_SEL_LO;
		for (n = 0; n < LCD_W; n++) {
			GLCD_Write(Framebuffer[n2++]);
		}
		GLCD_SEL_HI;
	}
}

#endif
#endif

