/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <stdint.h>
#include <rc.h>
#include <hd44100.h>
#include <spi_sw.h>

void LCD_Init(void) {
	LCD_DELAY1;
	LCD_DELAY1;
	LCD_DELAY1;
	LCD_DELAY1;
	LCD_DELAY1;
	LCD_DELAY1;
	LCD_DELAY1;
	LCD_DELAY1;
	LCD_DELAY1;

	LCD_Write(0x00, LCD_CMD);
	LCD_DELAY1;

	LCD_Write(32, LCD_CMD);
	LCD_DELAY1;
	LCD_DELAY1;

	LCD_Write(32, LCD_CMD);
	LCD_DELAY1;
	LCD_DELAY1;

	LCD_Write(40, LCD_CMD);
	LCD_DELAY1;
	LCD_DELAY1;

	LCD_Write(0, LCD_CMD);
	LCD_DELAY1;
	LCD_Write(LCD_ON, LCD_CMD);
	LCD_DELAY1;

	LCD_Write(0, LCD_CMD);
	LCD_DELAY1;
	LCD_Write(192, LCD_CMD);
	LCD_DELAY1;

}

void LCD_Write(uint8_t Data, uint8_t Rs) {
	uint8_t cmd = 0;

	LCD_Write_Port(0);

	if(Rs == 0) {
		cmd &= ~(1<<LCD_Pin_Rs);
	} else {
		cmd |= (1<<LCD_Pin_Rs);
	}

	if (Data & 16) {
		cmd |= (1<<LCD_Pin_Data_0);
	}
	if (Data & 32) {
		cmd |= (1<<LCD_Pin_Data_1);
	}
	if (Data & 64) {
		cmd |= (1<<LCD_Pin_Data_2);
	}
	if (Data & 128) {
		cmd |= (1<<LCD_Pin_Data_3);
	}
	cmd |= (1<<LCD_Pin_E);
	LCD_Write_Port(cmd);
	LCD_DELAY2;
	cmd &= ~(1<<LCD_Pin_E);
	LCD_Write_Port(cmd);
	LCD_DELAY2;

	cmd &= ~(1<<LCD_Pin_Data_0);
	cmd &= ~(1<<LCD_Pin_Data_1);
	cmd &= ~(1<<LCD_Pin_Data_2);
	cmd &= ~(1<<LCD_Pin_Data_3);
	if (Data & 1) {
		cmd |= (1<<LCD_Pin_Data_0);
	}
	if (Data & 2) {
		cmd |= (1<<LCD_Pin_Data_1);
	}
	if (Data & 4) {
		cmd |= (1<<LCD_Pin_Data_2);
	}
	if (Data & 8) {
		cmd |= (1<<LCD_Pin_Data_3);
	}
	cmd |= (1<<LCD_Pin_E);
	LCD_Write_Port(cmd);
	LCD_DELAY2;

	cmd &= ~(1<<LCD_Pin_E);
	LCD_Write_Port(cmd);
	LCD_DELAY2;
}

void LCD_Clear (void) {
	LCD_Write(LCD_CLEAR, LCD_CMD);
	LCD_DELAY1;
}

void LCD_Print (uint8_t zeile, uint8_t spalte, char *Buffer) {
	LCD_Write(zeile + spalte, LCD_CMD);
	while (*Buffer != 0) {
		LCD_Write(*Buffer++, LCD_CHAR);
	}
}

void LCD_Print_Matrix (char zeile, char spalte, char *matrix) {
	uint8_t n;
	LCD_Write(LCD_CGRAM(1), LCD_CMD);
	for (n = 0; n < 8; n++) {
		LCD_Write(matrix[n], LCD_CHAR);
	}
	LCD_Write(zeile + spalte, LCD_CMD);
	LCD_Write(1, LCD_CHAR);
}

void TitleScreen (void) {
	uint8_t n;
	char dat[] = {
		255,  0, 0, 0, 0, 0, 0, 0,
		0,  255, 0, 0, 0, 0, 0, 0,
		0,  0, 255, 0, 0, 0, 0, 0,
		0,  0, 0, 255, 0, 0, 0, 0,
		0,  0, 0, 0, 255, 0, 0, 0,
		0,  0, 0, 0, 0, 255, 0, 0,
		0,  0, 0, 0, 0, 0, 255, 0,
		0,  0, 0, 0, 0, 0, 0, 255,
	};
	LCD_Write(LCD_CGRAM(1), LCD_CMD);
	for (n = 0; n < sizeof(dat); n++) {
		LCD_Write(dat[n], LCD_CHAR);
	}
}

