/*
Copyright (C) 2008-2009  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include "rc.h"
#include "rc_adc.h"

#define ADC_NUM_CONVERTS	3
#define ADC_NUM_BUFFER		10

#ifdef PLATFORM_STM32
//#define ADC_SPEED ADC_SampleTime_55Cycles5
#define ADC_SPEED ADC_SampleTime_239Cycles5
//#define ADC1_DR_Address    ((u32)0x4001244C)
//uint16_t ADC_RegularConvertedValueTab[ADC_CHANNELS_MAX * ADC_NUM_CONVERTS];
//int16_t adc_slide[ADC_CHANNELS_MAX + 2][ADC_NUM_BUFFER + 2];
uint8_t ADC_CHANNEL_NUMS[] = ADC_INPUT_LIST;
#endif


#ifdef PLATFORM_SDL
extern uint16_t joy_value[20];
#endif

uint16_t GetADCChanel (uint8_t channel) {
#ifdef PLATFORM_STM32
/*
	uint32_t val = 0;
	uint32_t val2 = 0;
	uint8_t n = 0;
	for (n = 0; n < ADC_NUM_BUFFER - 1; n++) {
		adc_slide[channel][n] = adc_slide[channel][n + 1];
		val += adc_slide[channel][n];
	}
	ADC_RegularChannelConfig(ADC1, ADC_CHANNEL_NUMS[channel], 1, ADC_SampleTime_13Cycles5);
	for (n = 0; n < ADC_NUM_CONVERTS; n++) {
		ADC_SoftwareStartConvCmd(ADC1, ENABLE);
		while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
		val2 += ADC_GetConversionValue(ADC1);
	}
	adc_slide[channel][ADC_NUM_BUFFER - 1] = (val2 / ADC_NUM_CONVERTS);

	val += adc_slide[channel][ADC_NUM_BUFFER - 1];
	return (uint16_t)(val / ADC_NUM_BUFFER);
*/
	uint32_t val = 0;
	uint8_t n = 0;
	ADC_RegularChannelConfig(ADC1, ADC_CHANNEL_NUMS[channel], 1, ADC_SampleTime_13Cycles5);
	for (n = 0; n < 10; n++) {
		ADC_SoftwareStartConvCmd(ADC1, ENABLE);
		while(ADC_GetFlagStatus(ADC1, ADC_FLAG_EOC) == RESET);
		val += ADC_GetConversionValue(ADC1);
	}
	return (uint16_t)(val / 10);
#endif
#ifdef PLATFORM_SDL
	return (uint16_t)(joy_value[channel]);
#endif
}

int16_t Rc_AdcRead (int8_t id) {
	int32_t value = 0;
#ifdef PLATFORM_STM32
	if (id < 4) {
		value = (GetADCChanel(id) - 2048) + Rc_Calibrate_center[id];
		if (value > Rc_Calibrate_max[id]) {
			Rc_Calibrate_max[id] = value;
		}
		if (value < Rc_Calibrate_min[id]) {
			Rc_Calibrate_min[id] = value;
		}
		if (value > 0) {
			value = value * 1024 / Rc_Calibrate_max[id];
		} else if (value < 0) {
			value = value * 1024 / Rc_Calibrate_min[id] * -1;
		}
	} else if (id < ADC_CHANNELS_MAX) {
		value = (GetADCChanel(id) - 2048) / 2;
	}
	return value;
#endif
#ifdef PLATFORM_SDL
		return (GetADCChanel(id) - 2048) / 2;
#endif
}

void Rc_AdcInit (void) {
#ifdef PLATFORM_STM32
	GPIO_InitTypeDef GPIO_InitStructure;

        GPIO_InitStructure.GPIO_Pin = ADC_INPUT_PINS_A;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOA, &GPIO_InitStructure);

        GPIO_InitStructure.GPIO_Pin = ADC_INPUT_PINS_B;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOB, &GPIO_InitStructure);

        GPIO_InitStructure.GPIO_Pin = ADC_INPUT_PINS_C;
        GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AIN;
        GPIO_Init(GPIOC, &GPIO_InitStructure);

	ADC_InitTypeDef ADC_InitStructure;
	ADC_StructInit(&ADC_InitStructure);
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = DISABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = DISABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = 1;
	ADC_Init(ADC1, &ADC_InitStructure);
	ADC_Cmd(ADC1, ENABLE);


/* DMA-ADC init */
/* 
	DMA_InitTypeDef DMA_InitStructure;
	DMA_DeInit(DMA1_Channel1);
	DMA_InitStructure.DMA_PeripheralBaseAddr = ADC1_DR_Address;
	DMA_InitStructure.DMA_MemoryBaseAddr = (u32)ADC_RegularConvertedValueTab;
	DMA_InitStructure.DMA_DIR = DMA_DIR_PeripheralSRC;
	DMA_InitStructure.DMA_BufferSize = ADC_CHANNELS_MAX * 2;
	DMA_InitStructure.DMA_PeripheralInc = DMA_PeripheralInc_Disable;
	DMA_InitStructure.DMA_MemoryInc = DMA_MemoryInc_Enable;
	DMA_InitStructure.DMA_PeripheralDataSize = DMA_PeripheralDataSize_HalfWord;
	DMA_InitStructure.DMA_MemoryDataSize = DMA_MemoryDataSize_HalfWord;
	DMA_InitStructure.DMA_Mode = DMA_Mode_Circular;
	DMA_InitStructure.DMA_Priority = DMA_Priority_High;
	DMA_InitStructure.DMA_M2M = DMA_M2M_Disable;
	DMA_Init(DMA1_Channel1, &DMA_InitStructure);
	DMA_Cmd(DMA1_Channel1, ENABLE);

	ADC_InitTypeDef ADC_InitStructure;
	ADC_InitStructure.ADC_Mode = ADC_Mode_Independent;
	ADC_InitStructure.ADC_ScanConvMode = ENABLE;
	ADC_InitStructure.ADC_ContinuousConvMode = ENABLE;
	ADC_InitStructure.ADC_ExternalTrigConv = ADC_ExternalTrigConv_None;
	ADC_InitStructure.ADC_DataAlign = ADC_DataAlign_Right;
	ADC_InitStructure.ADC_NbrOfChannel = ADC_CHANNELS_MAX;
	ADC_Init(ADC1, &ADC_InitStructure);


#ifdef RCOS_MAINBOARD_V1_0
        //PA0
        ADC_RegularChannelConfig(ADC1, ADC_Channel_0 , 1, ADC_SPEED);
        //PA1
        ADC_RegularChannelConfig(ADC1, ADC_Channel_1 , 2, ADC_SPEED);
        //PA2
        ADC_RegularChannelConfig(ADC1, ADC_Channel_2 , 3, ADC_SPEED);
        //PA3
        ADC_RegularChannelConfig(ADC1, ADC_Channel_3 , 4, ADC_SPEED);
        //PA4
        ADC_RegularChannelConfig(ADC1, ADC_Channel_4 , 5, ADC_SPEED);
        //PA5
        ADC_RegularChannelConfig(ADC1, ADC_Channel_5 , 6, ADC_SPEED);
        //PA6
        ADC_RegularChannelConfig(ADC1, ADC_Channel_6 , 7, ADC_SPEED);
        //PA7
        ADC_RegularChannelConfig(ADC1, ADC_Channel_7 , 8, ADC_SPEED);
        //PB0
        ADC_RegularChannelConfig(ADC1, ADC_Channel_8 , 9, ADC_SPEED);
        //PB1
        ADC_RegularChannelConfig(ADC1, ADC_Channel_9 , 10, ADC_SPEED);
        //PC0
        ADC_RegularChannelConfig(ADC1, ADC_Channel_10 , 11, ADC_SPEED);
        //PC1
        ADC_RegularChannelConfig(ADC1, ADC_Channel_11 , 12, ADC_SPEED);
	//PC2
	ADC_RegularChannelConfig(ADC1, ADC_Channel_12 , 13, ADC_SPEED);
	//PC3
	ADC_RegularChannelConfig(ADC1, ADC_Channel_13 , 14, ADC_SPEED);
	//PC4
	ADC_RegularChannelConfig(ADC1, ADC_Channel_14 , 15, ADC_SPEED);
	// Temp
	ADC_RegularChannelConfig(ADC1, ADC_Channel_16 , 16, ADC_SPEED);
	ADC_TempSensorVrefintCmd(ENABLE);
#endif

#ifdef RCOS_MAINBOARD_FC16
	//PC4
	ADC_RegularChannelConfig(ADC1, ADC_Channel_14 , 1, ADC_SPEED);
        //PA1
        ADC_RegularChannelConfig(ADC1, ADC_Channel_1 , 2, ADC_SPEED);
        //PA0
        ADC_RegularChannelConfig(ADC1, ADC_Channel_0 , 3, ADC_SPEED);
	//PC2
	ADC_RegularChannelConfig(ADC1, ADC_Channel_12 , 4, ADC_SPEED);
	//PC3
	ADC_RegularChannelConfig(ADC1, ADC_Channel_13 , 5, ADC_SPEED);
        //PA4
        ADC_RegularChannelConfig(ADC1, ADC_Channel_4 , 6, ADC_SPEED);
        //PA2
        ADC_RegularChannelConfig(ADC1, ADC_Channel_2 , 7, ADC_SPEED);
        //PA5
        ADC_RegularChannelConfig(ADC1, ADC_Channel_5 , 8, ADC_SPEED);
        //PA6
        ADC_RegularChannelConfig(ADC1, ADC_Channel_6 , 9, ADC_SPEED);
        //PA7
        ADC_RegularChannelConfig(ADC1, ADC_Channel_7 , 10, ADC_SPEED);
        //PB0
        ADC_RegularChannelConfig(ADC1, ADC_Channel_8 , 11, ADC_SPEED);
        //PB1
        ADC_RegularChannelConfig(ADC1, ADC_Channel_9 , 12, ADC_SPEED);
        //PC0
        ADC_RegularChannelConfig(ADC1, ADC_Channel_10 , 13, ADC_SPEED);
        //PC1
        ADC_RegularChannelConfig(ADC1, ADC_Channel_11 , 14, ADC_SPEED);
        //PA3
        ADC_RegularChannelConfig(ADC1, ADC_Channel_3 , 15, ADC_SPEED);
	// Temp
	ADC_RegularChannelConfig(ADC1, ADC_Channel_16 , 16, ADC_SPEED);
	ADC_TempSensorVrefintCmd(ENABLE);
#endif

	ADC_DMACmd(ADC1, ENABLE);

	ADC_Cmd(ADC1, ENABLE);  
	ADC_ResetCalibration(ADC1);
	while(ADC_GetResetCalibrationStatus(ADC1));
	ADC_StartCalibration(ADC1);
	while(ADC_GetCalibrationStatus(ADC1));
	ADC_SoftwareStartConvCmd(ADC1, ENABLE);

*/

#endif

}

