/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <rc.h>
#include <rc_ppm-encoder.h>
#include <rc_adc.h>
#include <rc_control.h>

#ifdef SUPPORT_I2C

static I2C_InitTypeDef I2C_InitStructure;

void Rc_I2CInit (void) {
	GPIO_InitTypeDef  GPIO_InitStructure; 
	GPIO_InitStructure.GPIO_Pin =  GPIO_Pin_6 | GPIO_Pin_7;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_OD;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	I2C_InitStructure.I2C_Mode = I2C_Mode_I2C;
	I2C_InitStructure.I2C_DutyCycle = I2C_DutyCycle_2;
	I2C_InitStructure.I2C_OwnAddress1 = 111;
	I2C_InitStructure.I2C_Ack = I2C_Ack_Enable;
	I2C_InitStructure.I2C_AcknowledgedAddress = I2C_AcknowledgedAddress_7bit;
	I2C_InitStructure.I2C_ClockSpeed = 100000;
	I2C_Init(I2C1, &I2C_InitStructure);
	I2C_Cmd(I2C1, ENABLE);
}

void Rc_I2CWrite_test (void) {
	uint32_t timeout;
	static uint8_t count = 0;

	I2C_GenerateSTART(I2C1, ENABLE);
	timeout= 13000;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT) && timeout-- > 10);
	I2C_Send7bitAddress(I2C1, 64, I2C_Direction_Transmitter);
	timeout= 13000;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED) && timeout-- > 10);
	I2C_SendData(I2C1, count++);
	timeout= 13000;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED) && timeout-- > 10);
	I2C_GenerateSTOP(I2C1, ENABLE);
}

void Rc_I2CWrite_byte (uint8_t addr, uint8_t data) {
	uint16_t timeout;

	I2C_GenerateSTART(I2C1, ENABLE);
	timeout= 13000;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT) && timeout-- > 10);
	I2C_Send7bitAddress(I2C1, addr, I2C_Direction_Transmitter);
	timeout= 13000;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_TRANSMITTER_MODE_SELECTED) && timeout-- > 10);
	I2C_SendData(I2C1, data);
	timeout= 13000;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_TRANSMITTED) && timeout-- > 10);
	I2C_GenerateSTOP(I2C1, ENABLE);
}

void Rc_I2CRead_test (void) {
	uint32_t timeout;
	I2C_GenerateSTART(I2C1, ENABLE);
	timeout= 61000;
	while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_MODE_SELECT) && timeout-- > 1);
	if (timeout > 10) {
		I2C_Send7bitAddress(I2C1, 64, I2C_Direction_Receiver);
		timeout= 3000;
		while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_RECEIVER_MODE_SELECTED) && timeout-- > 1);
		if (timeout > 10) {
			I2C_GenerateSTOP(I2C1, DISABLE);
			timeout= 3000;
			while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED) && timeout-- > 1);
			if (timeout > 1) {
				Rc_Control_value[15] = I2C_ReceiveData(I2C1) - 128;
				I2C_GenerateSTOP(I2C1, DISABLE);
				timeout= 3000;
				while(!I2C_CheckEvent(I2C1, I2C_EVENT_MASTER_BYTE_RECEIVED) && timeout-- > 1);
				if (timeout > 1) {
					Rc_Control_value[16] = I2C_ReceiveData(I2C1) - 128;
					I2C_GenerateSTOP(I2C1, ENABLE);
				} else {
					I2C_DeInit(I2C1);
					I2C_Init(I2C1, &I2C_InitStructure);
					I2C_Cmd(I2C1, ENABLE);
				}
			} else {
				I2C_DeInit(I2C1);
				I2C_Init(I2C1, &I2C_InitStructure);
				I2C_Cmd(I2C1, ENABLE);
			}
		} else {
			I2C_DeInit(I2C1);
			I2C_Init(I2C1, &I2C_InitStructure);
			I2C_Cmd(I2C1, ENABLE);
		}
	} else {
		I2C_DeInit(I2C1);
		I2C_Init(I2C1, &I2C_InitStructure);
		I2C_Cmd(I2C1, ENABLE);
	}
}

void Rc_I2CRead_Data (uint8_t address, uint8_t *data, uint16_t len) {

}

void Rc_I2CWrite_Data (uint8_t address, uint8_t *data, uint16_t len) {

}

#endif


