/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include "rc.h"
#include "rc_ppm-encoder.h"

#ifdef SUPPORT_PPM1

void Rc_PPMEncoder1Set (uint8_t channel, int16_t value) {
	if (channel < Rc_PPMEncoder1_channels && value > -2048 && value < 2048) {
		Rc_PPMEncoder1_values[channel] = value * 6;
	}
}

void Rc_PPMEncoder1Power (uint8_t stat) {
	Rc_PPMEncoder1_power = stat;
	if (stat == 0) {
		GPIO_ResetBits(GPIOC, GPIO_Pin_8);
		GPIO_ResetBits(GPIOC, GPIO_Pin_9);
	} else {
		GPIO_SetBits(GPIOC, GPIO_Pin_8);
		GPIO_SetBits(GPIOC, GPIO_Pin_9);
	}
}

void Rc_PPMEncoder1Exit (void) {
	Rc_PPMEncoder1_power = 0;
	Rc_PPMEncoder1Power(0);
	TIM_Cmd(TIM3, DISABLE);
}

void Rc_PPMEncoder1Init (void) {
	uint8_t num;

	for (num = 0; num < PPM1_CHANNELS_MAX; num++) {
		Rc_PPMEncoder1_values[num] = 0;
	}

	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_8 | GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(GPIOC, &GPIO_InitStructure);
	GPIO_ResetBits(GPIOC, GPIO_Pin_8);
	GPIO_ResetBits(GPIOC, GPIO_Pin_9);


	TIM_OCInitTypeDef  TIM_OCInitStructure;
	vu16 CCR_Val = 65192;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM3, &TIM_TimeBaseStructure);
	TIM_PrescalerConfig(TIM3, 4, TIM_PSCReloadMode_Immediate);
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = CCR_Val;
	TIM_OC4Init(TIM3, &TIM_OCInitStructure);
	TIM_OC4PreloadConfig(TIM3, TIM_OCPreload_Disable);
	TIM_ITConfig(TIM3, TIM_IT_CC4, ENABLE);
	TIM_Cmd(TIM3, ENABLE);

	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = TIM3_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);
}



void TIM3_IRQHandler (void) {
	static uint8_t multichannel_counter[PPM1_MENCODER_MAX];
	uint8_t multichannel_num = 0;
	uint8_t multichannel_flag = 0;
	static uint8_t flag = 0;
	static uint8_t counter = 10;
	uint16_t capture = 0;
	if (TIM_GetITStatus(TIM3, TIM_IT_CC4) != RESET) {
		TIM_ClearITPendingBit(TIM3, TIM_IT_CC4);
		capture = TIM_GetCapture4(TIM3);

		// Puls
		if (flag == 0) {
			for (multichannel_num = 0; multichannel_num < PPM1_MENCODER_MAX; multichannel_num++) {
				if (counter == Rc_PPMEncoder1_Mencoder_channel[multichannel_num]) {
					// Multichannel Pulse
					if (Rc_PPMEncoder1_Mencoder_type[multichannel_num] == 0) {
						if (multichannel_counter[multichannel_num] < Rc_PPMEncoder1_Mencoder_max[multichannel_num]) {
							TIM_SetCompare4(TIM3, capture + (Rc_PPMEncoder1_center - Rc_PPMEncoder1_shift) - Rc_PPMEncoder1_Mencoder_values[multichannel_num][multichannel_counter[multichannel_num]]);
							multichannel_counter[multichannel_num]++;
						} else if (multichannel_counter[multichannel_num] < Rc_PPMEncoder1_Mencoder_max[multichannel_num] + 1) {
							TIM_SetCompare4(TIM3, capture + (Rc_PPMEncoder1_center - Rc_PPMEncoder1_shift) + 6000);
							multichannel_counter[multichannel_num]++;
						} else {
							TIM_SetCompare4(TIM3, capture + (Rc_PPMEncoder1_center - Rc_PPMEncoder1_shift) + 6000);
							multichannel_counter[multichannel_num] = 0;
						}
					}
					multichannel_flag = 1;
					counter++;
					break;
				}
			}

			if (multichannel_flag == 0) {
				// Channel-Pulse
				TIM_SetCompare4(TIM3, capture + (Rc_PPMEncoder1_center - Rc_PPMEncoder1_shift) + Rc_PPMEncoder1_values[counter++]);
			}

			if (Rc_PPMEncoder1_invert == 0) {
				GPIO_ResetBits(GPIOC, GPIO_Pin_8);
			} else {
				GPIO_SetBits(GPIOC, GPIO_Pin_8);
			}
			flag = 1;


		// Shift-Pulse
		} else if (flag == 1) {
			TIM_SetCompare4(TIM3, capture + Rc_PPMEncoder1_shift);
			if (Rc_PPMEncoder1_power != 0) {
				if (Rc_PPMEncoder1_invert == 0) {
					GPIO_SetBits(GPIOC, GPIO_Pin_8);
				} else {
					GPIO_ResetBits(GPIOC, GPIO_Pin_8);
				}
				GPIO_SetBits(GPIOC, GPIO_Pin_9);
			} else {
				GPIO_ResetBits(GPIOC, GPIO_Pin_9);
			}
			flag = 0;
		}

		// Sync-Pulse
		if (counter > Rc_PPMEncoder1_channels + 2) {
			TIM_SetCompare4(TIM3, capture + Rc_PPMEncoder1_sync);
			flag = 1;
			counter = 0;
			Rc_PPMEncoder1_frame_sync = 3;
		} else if (counter > Rc_PPMEncoder1_channels) {
			TIM_SetCompare4(TIM3, capture + Rc_PPMEncoder1_sync);
			flag = 2;
			counter++;
			Rc_PPMEncoder1_frame_sync = 1;
		}
	}
}

#endif


#ifdef SUPPORT_PPM2

void Rc_PPMEncoder2Set (uint8_t channel, int16_t value) {
	if (channel < Rc_PPMEncoder2_channels && value > -2048 && value < 2048) {
		Rc_PPMEncoder2_values[channel] = value * 6;
	}
}

void Rc_PPMEncoder2Power (uint8_t stat) {
	Rc_PPMEncoder2_power = stat;
	if (stat == 0) {
	} else {
	}
}

void Rc_PPMEncoder2Exit (void) {
	Rc_PPMEncoder2_power = 0;
	Rc_PPMEncoder2Power(0);
	TIM_Cmd(TIM2, DISABLE);
}

void Rc_PPMEncoder2Init (void) {
	uint8_t num;
	Rc_PPMEncoder2_power = 1;
	for (num = 0; num < PPM2_CHANNELS_MAX; num++) {
		Rc_PPMEncoder2_values[num] = 0;
	}



	TIM_OCInitTypeDef  TIM_OCInitStructure;
	vu16 CCR_Val = 65192;
	TIM_TimeBaseInitTypeDef TIM_TimeBaseStructure;
	TIM_TimeBaseStructure.TIM_Period = 65535;
	TIM_TimeBaseStructure.TIM_Prescaler = 0;
	TIM_TimeBaseStructure.TIM_ClockDivision = 0;
	TIM_TimeBaseStructure.TIM_CounterMode = TIM_CounterMode_Up;
	TIM_TimeBaseInit(TIM2, &TIM_TimeBaseStructure);
	TIM_PrescalerConfig(TIM2, 4, TIM_PSCReloadMode_Immediate);
	TIM_OCInitStructure.TIM_OutputState = TIM_OutputState_Enable;
	TIM_OCInitStructure.TIM_Pulse = CCR_Val;
	TIM_OC4Init(TIM2, &TIM_OCInitStructure);
	TIM_OC4PreloadConfig(TIM2, TIM_OCPreload_Disable);
	TIM_ITConfig(TIM2, TIM_IT_CC4, ENABLE);
	TIM_Cmd(TIM2, ENABLE);

	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = TIM2_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 0;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 1;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

}

void TIM2_IRQHandler (void) {
	static uint8_t flag = 0;
	static uint8_t counter = 10;
	uint16_t capture;
	if (TIM_GetITStatus(TIM2, TIM_IT_CC4) != RESET) {
		TIM_ClearITPendingBit(TIM2, TIM_IT_CC4);
		capture = TIM_GetCapture4(TIM2);
		if (flag == 0) {
			// Channel-Pulse
			TIM_SetCompare4(TIM2, capture + (Rc_PPMEncoder2_center - Rc_PPMEncoder2_shift) + Rc_PPMEncoder2_values[counter++]);
			if (Rc_PPMEncoder1_invert == 0) {
			} else {
			}
			flag = 1;
		} else if (flag == 1) {
			// Shift-Pulse
			TIM_SetCompare4(TIM2, capture + Rc_PPMEncoder2_shift);
			if (Rc_PPMEncoder2_power != 0) {
				if (Rc_PPMEncoder1_invert == 0) {
				} else {
				}
			} else {
			}
			if (counter == 0) {
			} else {
			}
			flag = 0;
		}
		// Sync-Pulse
		if (counter >= Rc_PPMEncoder2_channels + 3) {
			TIM_SetCompare4(TIM2, capture + Rc_PPMEncoder2_sync);
			flag = 1;
			counter = 0;
		} else if (counter >= Rc_PPMEncoder2_channels + 2) {
			TIM_SetCompare4(TIM2, capture + Rc_PPMEncoder2_sync);
			flag = 2;
			counter++;
		} else if (counter >= Rc_PPMEncoder2_channels + 1) {
			TIM_SetCompare4(TIM2, capture + Rc_PPMEncoder2_sync);
			flag = 0;
			counter++;
		}
	}
}

#endif
