/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <rc.h>
#include <rc_ppm-encoder.h>
#include <rc_adc.h>
#include <rc_control.h>

#define SHIFTREG_SPEED	2

#ifdef PLATFORM_STM32
#define SHIFTREG_PORT	GPIOA
#define SHIFTREG_CLK	GPIO_Pin_11
#define SHIFTREG_P2S	GPIO_Pin_12
#define SHIFTREG_SINR	GPIO_Pin_13
#define SHIFTREG_SINL	GPIO_Pin_14
#endif


void Rc_ShiftRegInit (void) {
#ifdef PLATFORM_STM32
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = SHIFTREG_SINL | SHIFTREG_SINR;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(SHIFTREG_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = SHIFTREG_CLK | SHIFTREG_P2S;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(SHIFTREG_PORT, &GPIO_InitStructure);

	GPIO_ResetBits(SHIFTREG_PORT, SHIFTREG_CLK);
	GPIO_ResetBits(SHIFTREG_PORT, SHIFTREG_P2S);
#endif
}

void Rc_ShiftRegRead_Data (uint8_t *data, uint16_t len) {
	uint16_t timeout = 0;
	uint8_t num = 0;
	uint16_t n = 0;

#ifdef PLATFORM_STM32

	// par -> ser
	GPIO_SetBits(SHIFTREG_PORT, SHIFTREG_P2S);
	//timeout = SHIFTREG_SPEED; while(timeout-- > 1);

	GPIO_SetBits(SHIFTREG_PORT, SHIFTREG_CLK);
	//timeout = SHIFTREG_SPEED; while(timeout-- > 1);
	GPIO_ResetBits(SHIFTREG_PORT, SHIFTREG_CLK);
	//timeout = SHIFTREG_SPEED; while(timeout-- > 1);

	GPIO_ResetBits(SHIFTREG_PORT, SHIFTREG_P2S);
	//timeout = SHIFTREG_SPEED; while(timeout-- > 1);

	// read data
	for (n = 0; n < (len / 2); n++) {
		for (num = 0; num < 8; num++) {
			if( GPIO_ReadInputDataBit(SHIFTREG_PORT, SHIFTREG_SINL) != 0 ) {
				data[n] |= (1<<num);
			} else {
				data[n] &= ~(1<<num);
			}
			if( GPIO_ReadInputDataBit(SHIFTREG_PORT, SHIFTREG_SINR) != 0 ) {
				data[n + (len / 2)] |= (1<<num);
			} else {
				data[n + (len / 2)] &= ~(1<<num);
			}
			GPIO_SetBits(SHIFTREG_PORT, SHIFTREG_CLK);
			//timeout = SHIFTREG_SPEED; while(timeout-- > 1);
			GPIO_ResetBits(SHIFTREG_PORT, SHIFTREG_CLK);
			//timeout = SHIFTREG_SPEED; while(timeout-- > 1);
		}
	}
#endif
}

void Rc_ShiftRegWrite_Data (uint8_t *data, uint16_t len) {

}



