/*
Copyright (C) 2008-2009  Oliver Dippel <o.dippel@gmx.de>

The SD/MMC Init-Sequenz is original written for AVR's by : Holger Klabunde <hk@holger>
	(http://www.holger-klabunde.de/index.html)

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <stdint.h>
#include <rc.h>
#include <spi_sw.h>
#include <sd.h>
#include <lcd.h>

uint8_t mmc_card_type = MMC_CARD_NONE;
uint8_t mmc_card_present = 0;
uint32_t mmc_maxsect;

#ifdef SUPPORT_MMC

#ifdef PLATFORM_STM32
void mmc_read_buffer (uint8_t *buffer, uint8_t numbytes) {
	uint8_t n = 0;
	for (n = 0; n < numbytes; n++) {
		*buffer++ = MMC_SPI_Read_Byte();
	}
}

uint8_t mmc_send_cmd (uint8_t command, unsigned long adress) {
	uint8_t tmp = 0xff;
	uint16_t Timeout = 0;
	MMC_SPI_Disable();
	MMC_SPI_Write_Byte(0xFF);
	MMC_SPI_Enable();
	MMC_SPI_Write_Byte(command);
	MMC_SPI_Write_Byte((adress & 0xFF000000)>>24);
	MMC_SPI_Write_Byte((adress & 0x00FF0000)>>16);
	MMC_SPI_Write_Byte((adress & 0x0000FF00)>>8);
	MMC_SPI_Write_Byte((adress & 0x000000FF));
	MMC_SPI_Write_Byte(0x95);
	while (tmp == 0xff) {
		if (Timeout++ > 500) {
			break;
		}
		tmp = MMC_SPI_Read_Byte();
	}
	return tmp;
}
#endif

uint8_t mmc_init (void) {
#ifdef PLATFORM_STM32
	uint16_t n;
	uint8_t ret;
	uint8_t csd_version; 
	uint8_t ret_buffer[16];
	mmc_card_type = MMC_CARD_NONE; 
	MMC_SPI_Slow();
	for (n = 0; n < 10; n++) {
		MMC_SPI_Write_Byte(0xFF);
	}                                      
	for (n = 0; n < 10; n++) {
		if (mmc_send_cmd(MMC_CMD0, 0) == MMC_IDLE_STATE) {
			break;
		}
	}  
	if (ret != MMC_IDLE_STATE) {
		MMC_Disable();
		BootMSG(255, "MMC..IDLE");
		return MMC_CMD0_TIMEOUT;
	}
	mmc_card_type = MMC_CARD_UNKNOWN; 
	if ((mmc_send_cmd(MMC_CMD8, (unsigned long)0x00000122) & MMC_ILLEGAL_COMMAND) == 0) {
		mmc_card_type = MMC_CARD_SD;
		mmc_read_buffer(ret_buffer, 4);
		if (ret_buffer[2] != 0x01) {
			MMC_Disable();
			BootMSG(255, "SD-Card..VOLTAGE_ERROR");
			return MMC_VOLTAGE_ERROR;
		}
		if (ret_buffer[3] != 0x22) {
			MMC_Disable();
			BootMSG(255, "SD-Card..PATTERN_ERROR");
			return MMC_PATTERN_ERROR;
		}
		mmc_send_cmd(MMC_CMD58, 0);
		mmc_read_buffer(ret_buffer, 4);
		n = 0;
		while (mmc_send_cmd(MMC_CMD41, 0x40000000) != 0) {
			if (n++ > 1024) {
				MMC_Disable();
				BootMSG(255, "SD-Card...Timeout");
				return MMC_CMD41_TIMEOUT;
			}
			mmc_send_cmd(MMC_CMD55, 0);
		}
		ret = mmc_send_cmd(MMC_CMD58, 0);
		mmc_read_buffer(ret_buffer, 4);
		if (ret_buffer[0] & 0x40) {
			mmc_card_type = MMC_CARD_SDHC;
		}
	} else {
		if ((mmc_send_cmd(MMC_CMD55,0) & MMC_ILLEGAL_COMMAND) == 0) {
			mmc_card_type = MMC_CARD_SD;
			n = 0;
			while (mmc_send_cmd(MMC_CMD41,0) != 0) {
				if (n++ > 1024) {
					MMC_Disable();
					BootMSG(255, "CMD41_TIMEOUT");
					return MMC_CMD41_TIMEOUT;
				}
				mmc_send_cmd(MMC_CMD55, 0);
			}
		} else {
			mmc_card_type = MMC_CARD_MMC;
			n = 0;
			while (mmc_send_cmd(MMC_CMD1, 0) != 0) {
				if (n++ > 1024) {
					MMC_Disable();
					BootMSG(255, "MMC-Card..Timeout");
					return MMC_CMD1_TIMEOUT;
				}
			}
		}
	}
	ret = mmc_send_cmd(MMC_READ_CSD, 0);
	n = 0;
	while (ret != MMC_START_BLOCK_TOKEN) {
		if (n++ > 1024) {
			break;
		}
		ret = MMC_SPI_Write_Byte(0xFF);
	}
	mmc_read_buffer(ret_buffer, 16);
	MMC_SPI_Write_Byte(0xFF);
	MMC_SPI_Write_Byte(0xFF);
	MMC_Disable();
	if (mmc_card_type == MMC_CARD_MMC) {
		csd_version = 0;
	} else {
		csd_version = ret_buffer[0] >> 6;
	}
	if (csd_version == 2 || csd_version == 3) {
		BootMSG(255, "MMC..CSD_ERROR");
	} else {
		if (csd_version == 0) {
//			mmc_maxsect = (1<<(ret_buffer[5] & 0x0F)) / MMC_BYTE_PER_SEC * (unsigned long)((((uint16_t)(ret_buffer[6] & 0x03)<<10) + ((uint16_t)ret_buffer[7]<<2) + ret_buffer[8]>>6) + 1) * (unsigned long)(1<<((2 + ((ret_buffer[9] & 0x03)<<1) + (ret_buffer[10]>>7))));
			mmc_maxsect = (1<<(ret_buffer[5] & 0x0F)) / MMC_BYTE_PER_SEC * (unsigned long)(((uint16_t)(ret_buffer[6] & 0x03)<<10 + (uint16_t)ret_buffer[7]<<2 + ret_buffer[8]>>6) + 1) * (unsigned long)(1<<(2 + ((ret_buffer[9] & 0x03)<<1 + (ret_buffer[10]>>7))));
		} else {
//			mmc_maxsect = 1024 * ((((ret_buffer[7] & 0x3F)<<16) + ((uint16_t)ret_buffer[8]<<8) + ret_buffer[9]) + 1);
			mmc_maxsect = 1024 * (((ret_buffer[7] & 0x3F)<<16 + (uint16_t)ret_buffer[8]<<8 + ret_buffer[9]) + 1);
		}
	}
	BootMSG(100, "Card...done");
	BootMSG(255, "MMC..All right");
	return MMC_OK;
#endif
#ifdef PLATFORM_SDL
	system("mkdir -p /tmp/rcos-mmc/");
	return MMC_OK;
#endif
}

uint8_t mmc_read_sector (unsigned long sector, uint8_t *buffer) {
#ifdef PLATFORM_STM32
	uint32_t n;
	uint8_t ret;
	unsigned long startadr;
	if (mmc_card_type == MMC_CARD_NONE || sector >= mmc_maxsect) {
		return 1;
	}
//	MMC_SPI_Slow();
	startadr = sector;
	if (mmc_card_type != MMC_CARD_SDHC) {
		startadr *= MMC_BYTE_PER_SEC;
	}
	ret = mmc_send_cmd(MMC_READ_BLOCK, startadr);
	while (ret != MMC_START_BLOCK_TOKEN) {
		ret = MMC_SPI_Write_Byte(0xFF);
	}
	for (n = 0; n < MMC_BYTE_PER_SEC; n++) {
		*buffer++ = MMC_SPI_Write_Byte(0xFF);
	}
	MMC_SPI_Write_Byte(0xFF);
	MMC_SPI_Write_Byte(0xFF);
	MMC_Disable();
	return 0;
#endif
#ifdef PLATFORM_SDL
	char tmp_str[100];
	FILE *stream = NULL;
	sprintf(tmp_str, "/tmp/rcos-mmc/sector.%i", sector);
	if ((stream = fopen(tmp_str, "r")) != 0) {
		fread(buffer, 1, MMC_BYTE_PER_SEC, stream);
		fclose(stream);
	}
	return 0;
#endif
}

uint8_t mmc_write_sector (unsigned long sector, uint8_t *buffer) {
	uint32_t n;
#ifdef PLATFORM_STM32
	unsigned long startadr;
	if (mmc_card_type == MMC_CARD_NONE || sector >= mmc_maxsect) {
		return 1;
	}
//	MMC_SPI_Slow();
	startadr = sector;
	if (mmc_card_type != MMC_CARD_SDHC) {
		startadr *= MMC_BYTE_PER_SEC;
	}
	mmc_send_cmd(MMC_WRITE_BLOCK, startadr);
	for (n = 0; n < 100; n++) {
		MMC_SPI_Read_Byte();
	}
	MMC_SPI_Write_Byte(MMC_START_BLOCK_TOKEN);
	for (n = 0; n < MMC_BYTE_PER_SEC; n++) {
		MMC_SPI_Write_Byte(*buffer++);
	}
	MMC_SPI_Write_Byte(0xFF);
	MMC_SPI_Write_Byte(0xFF);
	n = 0;
	while (MMC_SPI_Read_Byte() != 0xff){
		if (n++ > 4096) {
			MMC_Disable();
			BootMSG(255, "MMC..WRITE_TIMEOUT");
			return 1;
		}
	}
	MMC_Disable();
	return 0;
#endif
#ifdef PLATFORM_SDL
	char tmp_str[100];
	FILE *stream = NULL;
	sprintf(tmp_str, "/tmp/rcos-mmc/sector.%i", sector);
	if ((stream = fopen(tmp_str, "w")) != 0) {
		fwrite(buffer, 1, MMC_BYTE_PER_SEC, stream);
		fclose(stream);
	}
	return 0;
#endif
}

#endif

