
#include <rc.h>
#include <stdint.h>
#include <spi_sw.h>

#ifdef SUPPORT_SPI

#ifdef SUPPORT_HWSPI
uint8_t spi_mode = 1;
#else
uint8_t spi_mode = 0;
#endif

uint16_t spi_speed = SPI_BaudRatePrescaler_2;

void delay (uint32_t time) {
	uint32_t n;
	for (n = 0; n < time; n++) {
		asm volatile ("nop");
	}
}

void SPI_HWInit (void) {
	GPIO_InitTypeDef GPIO_InitStructure;
	if (spi_mode == 0) {
		GPIO_InitStructure.GPIO_Pin = SPI_MOSI_PIN;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_Init(SPI_MOSI_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = SPI_MISO_PIN;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_Init(SPI_MISO_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = SPI_CLK_PIN;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_Init(SPI_CLK_PORT, &GPIO_InitStructure);
	} else {
		GPIO_InitStructure.GPIO_Pin = SPI_MOSI_PIN;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(SPI_MOSI_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = SPI_MISO_PIN;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(SPI_MISO_PORT, &GPIO_InitStructure);

		GPIO_InitStructure.GPIO_Pin = SPI_CLK_PIN;
		GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
		GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
		GPIO_Init(SPI_CLK_PORT, &GPIO_InitStructure);

		SPI_Cmd(HWSPI, DISABLE);
		SPI_InitTypeDef   SPI_InitStructure;
		SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
		SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
		SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
		SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
		SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
		SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
		SPI_InitStructure.SPI_BaudRatePrescaler = spi_speed;
		SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
		SPI_InitStructure.SPI_CRCPolynomial = 7;
		SPI_Init(HWSPI, &SPI_InitStructure);
		SPI_Cmd(HWSPI, ENABLE);
	}

	GPIO_InitStructure.GPIO_Pin = SPI_SEL1_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(SPI_SEL1_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = SPI_SEL2_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(SPI_SEL2_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = SPI_SEL3_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(SPI_SEL3_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = SPI_AD_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(SPI_AD_PORT, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = SPI_RST_PIN;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_Init(SPI_RST_PORT, &GPIO_InitStructure);

	if (spi_mode == 0) {
		SPI_MOSI_LO();
		SPI_CLK_LO();
		SPI_SEL1_LO();
		SPI_SEL2_LO();
		SPI_SEL3_LO();
		SPI_AD_HI();
		SPI_RST_HI();
	}
}

void SPI_Mode (uint8_t mode) {
	if (spi_mode != mode) {
		spi_mode = mode;
		SPI_HWInit();
	}
}

void SPI_Speed (uint16_t speed) {
	if (spi_mode == 1 && spi_speed != speed) {
		SPI_Cmd(HWSPI, DISABLE);
		SPI_InitTypeDef   SPI_InitStructure;
		SPI_InitStructure.SPI_Direction = SPI_Direction_2Lines_FullDuplex;
		SPI_InitStructure.SPI_Mode = SPI_Mode_Master;
		SPI_InitStructure.SPI_DataSize = SPI_DataSize_8b;
		SPI_InitStructure.SPI_CPOL = SPI_CPOL_High;
		SPI_InitStructure.SPI_CPHA = SPI_CPHA_2Edge;
		SPI_InitStructure.SPI_NSS = SPI_NSS_Soft;
		SPI_InitStructure.SPI_BaudRatePrescaler = speed;
		SPI_InitStructure.SPI_FirstBit = SPI_FirstBit_MSB;
		SPI_InitStructure.SPI_CRCPolynomial = 7;
		SPI_Init(HWSPI, &SPI_InitStructure);
		SPI_Cmd(HWSPI, ENABLE);
	}
	spi_speed = speed;
}

void SPI_Fast (void) {
	SPI_Speed(SPI_BaudRatePrescaler_2);
}

void SPI_Slow (void) {
	SPI_Speed(SPI_BaudRatePrescaler_256);
}

uint8_t SPI_Write_Byte (uint8_t data) {
	uint8_t temp = 0;
	if (spi_mode == 0) {
		uint8_t i = 0;
		SPI_CLK_LO();
		for(i = 0; i < 8; i++){
			temp <<= 1;
			if (SPI_MISO_HI()){
				temp |= 0x01;
			}

			SPI_CLK_LO();
			if (data & 0x80){
				SPI_MOSI_HI();
			} else {
				SPI_MOSI_LO();
			}
			SPI_CLK_HI();
			data <<= 1;
		};
		SPI_CLK_LO();
		SPI_MOSI_HI();
	} else {
		SPI_I2S_SendData(HWSPI, data);
		while (SPI_I2S_GetFlagStatus(HWSPI, SPI_I2S_FLAG_TXE) == RESET);
		temp = SPI_I2S_ReceiveData(HWSPI);
	}
	return temp;
}

void SPI_Write_Word (uint16_t data) {
	SPI_Write_Byte((data>>8));
	SPI_Write_Byte(data);
}

uint8_t SPI_Read_Byte (void) {
	uint8_t temp = 0;
	uint8_t data = 0xff;
	if (spi_mode == 0) {
		uint8_t i = 0;
		SPI_CLK_LO();
		for(i = 0; i < 8; i++){
			temp <<= 1;
			if (SPI_MISO_HI()){
				temp |= 0x01;
			}
			SPI_CLK_HI();
			if (data & 0x80){
				SPI_MOSI_HI();
			} else {
				SPI_MOSI_LO();
			}
			SPI_CLK_LO();
			data <<= 1;
		};
		SPI_CLK_LO();
		SPI_MOSI_HI();
	} else {
		SPI_I2S_SendData(HWSPI, data);
		while (SPI_I2S_GetFlagStatus(HWSPI, SPI_I2S_FLAG_TXE) == RESET);
		while (SPI_I2S_GetFlagStatus(HWSPI, SPI_I2S_FLAG_RXNE) == RESET);
		temp = SPI_I2S_ReceiveData(HWSPI);
	}
	return temp;
}


void SPI_Select1_HI (void) {
	SPI_SEL1_HI();

}

void SPI_Select1_LO (void) {
	SPI_SEL1_LO();

}

void SPI_Select2_HI (void) {
	SPI_SEL2_HI();

}

void SPI_Select2_LO (void) {
	SPI_SEL2_LO();

}

void SPI_Select3_HI (void) {
	SPI_SEL3_HI();

}

void SPI_Select3_LO (void) {
	SPI_SEL3_LO();

}

void SPI_AddressData_HI (void) {
	SPI_AD_HI();

}

void SPI_AddressData_LO (void) {
	SPI_AD_LO();

}

void SPI_Reset_HI (void) {
	SPI_RST_HI();

}

void SPI_Reset_LO (void) {
	SPI_RST_LO();

}

void SPI_74xx595_Write (uint8_t data) {
	uint8_t n;
	for (n = 0; n < 8; n++) {
		if (data & (1<<n)) {
			GPIO_SetBits(SPI_MOSI_PORT, SPI_MOSI_PIN);
		} else {
			GPIO_ResetBits(SPI_MOSI_PORT, SPI_MOSI_PIN);
		}
		GPIO_SetBits(SPI_CLK_PORT, SPI_CLK_PIN);
		delay(10);
		GPIO_ResetBits(SPI_CLK_PORT, SPI_CLK_PIN);
	}
	GPIO_SetBits(SPI_SEL1_PORT, SPI_SEL1_PIN);
	delay(10);
	GPIO_ResetBits(SPI_SEL1_PORT, SPI_SEL1_PIN);
}

#endif

