/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <string.h>
#include <rc.h>
#include <sd.h>
#include <rc_control.h>

#ifdef SUPPORT_USART

volatile uint8_t TxBuffer1[1024];
volatile uint8_t RxBuffer1[1024];
volatile uint8_t Tx1_Flag = 0;
volatile uint8_t Tx1_Start = 0;
volatile uint16_t TxCounter1 = 0;
volatile uint16_t NbrOfDataToTransfer1 = 0;


void init_usart1 (void) {

	/* Configure USART1 Rx (PA.10) as input floating */
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Configure USART1 Tx (PA.09) as alternate function push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_9;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOA, &GPIO_InitStructure);

	/* Enable the USART1 Interrupt */
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = USART1_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 4;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 4;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = 115200;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_Init(USART1, &USART_InitStructure);

	USART_ITConfig(USART1, USART_IT_RXNE, ENABLE);
	USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
	USART_Cmd(USART1, ENABLE);

}

void USART1_SEND (char *data, uint16_t count) {
	memcpy(&TxBuffer1, data, count);
	NbrOfDataToTransfer1 = count;
	TxCounter1 = 0;
	Tx1_Start = 1;
}

void USART1_SEND_STR (char *data) {
	uint16_t count = strlen(data);
	memcpy(&TxBuffer1, data, count);
	NbrOfDataToTransfer1 = count;
	TxCounter1 = 0;
	Tx1_Start = 1;
}

void USART1_TX_START (void) {
	TxCounter1 = 0;
	Tx1_Start = 1;
}

void USART1_SEND_OK (void) {
	USART1_SEND("OK\n", 3);
}

void USART1_TX_START_OK (void) {
	if (Tx1_Start == 1) {
		Tx1_Start = 0;
		USART_ITConfig(USART1, USART_IT_TXE, ENABLE);
	}
}

void USART1_IRQHandler (void) {
	static uint8_t data_csum = 0;
	static uint8_t cmd_mode = 0;
	static uint16_t cmd_counter = 0;
	uint8_t csum;
	uint8_t RX;
	if(USART_GetITStatus(USART1, USART_IT_TXE) != RESET) {
		USART_SendData(USART1, TxBuffer1[TxCounter1++]);
		if(TxCounter1 == NbrOfDataToTransfer1) {
			USART_ITConfig(USART1, USART_IT_TXE, DISABLE);
			NbrOfDataToTransfer1 = 0;
			TxCounter1 = 0;
		}    
	}
	if(USART_GetITStatus(USART1, USART_IT_RXNE) != RESET) {
		RX = USART_ReceiveData(USART1);
		switch (cmd_mode) {
			case FE_CMD_NONE:
				if (RX < FE_CMD_LAST) {
					cmd_mode = RX;
					data_csum = 255 ^ cmd_mode;
					cmd_counter = 0;
				} else {
					Tx1_Flag = 254;
					cmd_mode = FE_CMD_NONE;
					cmd_counter = 0;
				}
			break;
			case FE_CMD_SERVO_TEST:
				if (cmd_counter < 1) {
					RxBuffer1[cmd_counter++] = RX;
					data_csum ^= RX;
				} else {
					csum = RX;
					if (csum == data_csum) {
						Servo_Testflag = RxBuffer1[0] + 1;
						Tx1_Flag = 253;
					} else {
						Tx1_Flag = 255;
					}
					cmd_mode = FE_CMD_NONE;
				}
			break;
			case FE_CMD_CONTROL_VALUE_SET:
			case FE_CMD_CONTROL_SET:
			case FE_CMD_PPMENCODER1_MENCODER_SET:
			case FE_CMD_PPMENCODER1_SET:
			case FE_CMD_PPMENCODER2_SET:
			case FE_CMD_OUTPUT_SET:
			case FE_CMD_EXPO_SET:
			case FE_CMD_LIMITER_SET:
			case FE_CMD_ATV_SET:
			case FE_CMD_DUALRATE_SET:
			case FE_CMD_MOTION_SET:
			case FE_CMD_MIXER_SET:
				if (cmd_counter < FeCmdLen[cmd_mode]) {
					RxBuffer1[cmd_counter++] = RX;
					data_csum ^= RX;
				} else {
					csum = RX;
					if (csum == data_csum) {
						Tx1_Flag = cmd_mode;
					} else {
						Tx1_Flag = 255;
					}
					cmd_mode = FE_CMD_NONE;
				}
			break;
			default:
				csum = RX;
				if (csum == data_csum) {
					Tx1_Flag = cmd_mode;
				} else {
					Tx1_Flag = 255;
				}
				cmd_mode = FE_CMD_NONE;
			break;
		}
	}
}

#endif
