/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <string.h>
#include <rc.h>
#include <sd.h>
#include <rc_control.h>

#ifdef SUPPORT_USART

uint8_t TxBuffer3[100];
volatile uint16_t TxCounter3 = 0;
uint16_t NbrOfDataToTransfer3 = 0;

extern void USART1_SEND (char *data, uint16_t count);
extern void USART3_SEND (char *data, uint16_t count);
extern void USART3_SEND_STR (char *data);


void init_usart3 (void) {
	/* Configure USART3 Rx (PB.11) as input floating */
	GPIO_InitTypeDef GPIO_InitStructure;
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_11;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IN_FLOATING;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	/* Configure USART3 Tx (PB.10) as alternate function push-pull */
	GPIO_InitStructure.GPIO_Pin = GPIO_Pin_10;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_AF_PP;
	GPIO_Init(GPIOB, &GPIO_InitStructure);

	/* Enable the USART3 Interrupt */
	NVIC_InitTypeDef NVIC_InitStructure;
	NVIC_InitStructure.NVIC_IRQChannel = USART3_IRQChannel;
	NVIC_InitStructure.NVIC_IRQChannelPreemptionPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelSubPriority = 3;
	NVIC_InitStructure.NVIC_IRQChannelCmd = ENABLE;
	NVIC_Init(&NVIC_InitStructure);

	USART_InitTypeDef USART_InitStructure;
	USART_InitStructure.USART_BaudRate = 38400;
	USART_InitStructure.USART_WordLength = USART_WordLength_8b;
	USART_InitStructure.USART_StopBits = USART_StopBits_1;
	USART_InitStructure.USART_Parity = USART_Parity_No;
	USART_InitStructure.USART_HardwareFlowControl = USART_HardwareFlowControl_None;
	USART_InitStructure.USART_Mode = USART_Mode_Rx | USART_Mode_Tx;
	USART_Init(USART3, &USART_InitStructure);

	USART_ITConfig(USART3, USART_IT_RXNE, ENABLE);
	USART_ITConfig(USART3, USART_IT_TXE, ENABLE);
	USART_Cmd(USART3, ENABLE);

}

void USART3_SEND (char *data, uint16_t count) {
	memcpy(&TxBuffer3, data, count);
	NbrOfDataToTransfer3 = count;
	TxCounter3 = 0;
	USART_ITConfig(USART3, USART_IT_TXE, ENABLE);
}


/*
typedef struct {
	uint8_t mode;
	uint8_t input;
	uint8_t max;
	uint8_t min;
} ActDSL_ServoMixer;

typedef struct {
	uint8_t id;
	uint16_t pulse_max;
	uint16_t pulse_min;
	ActDSL_ServoMixer mixer[4];
	uint8_t delay_max;
	uint8_t delay_min;
} ActDSL_Servo;

typedef struct {
	uint8_t freq; // FREQ
	uint8_t mode; // 0=Diversity, 1=Trainer, 2=Diversity 1RX, 3=Trainer 1RX (4 = TS)
	uint8_t trainer_freq; // Trainer-Freq
	uint8_t trainer_sw; // Trainer-SW
	uint8_t tx[12]; // 48=OFF, 49=ON
	uint8_t enc; // 10=PPM, 100=PCM1024, 20=SPCM20
} ActDSL_Setup;

typedef struct {
	uint8_t unknown1;
	uint8_t ch[48];
} ActDSL_Scann;

typedef struct {
	uint8_t level;
	uint8_t tmp0;
	uint8_t voltage;
	uint8_t tmp1;
	uint8_t ch1;
	uint8_t tmp2;
	uint8_t ch2;
	uint8_t tmp3;
	uint8_t ch3;
	uint8_t tmp4;
	uint8_t ch4;
	uint8_t tmp5;
	uint8_t ch5;
	uint8_t tmp6;
	uint8_t ch6;
	uint8_t tmp7;
	uint8_t ch7;
	uint8_t tmp8;
	uint8_t ch8;
	uint8_t tmp9;
	uint8_t ch9;
	uint8_t tmp10;
	uint8_t ch10;
	uint8_t tmp11;
	uint8_t tmp12;
	uint8_t tmp13;
	uint8_t tmp14;
} ActDSL_Info;

ActDSL_Servo servo;
ActDSL_Setup setup;
ActDSL_Scann scann;
ActDSL_Info info;

*/


void reverse(char s[]) {
	int c, i, j;
	for (i = 0, j = strlen(s)-1; i<j; i++, j--) {
		c = s[i];
		s[i] = s[j];
		s[j] = c;
	}
}


void itoa(int n, char s[]) {
	int i, sign;
	
	if ((sign = n) < 0) {
		n = -n;
	}
	i = 0;
	do {
		s[i++] = n % 10 + '0';
	} while ((n /= 10) > 0);
	if (sign < 0) {
		s[i++] = '-';
	}
	s[i] = '\0';
	reverse(s);
} 


void Act_Cmd_Send(uint8_t cmd) {
	TxBuffer3[0] = cmd;
	TxBuffer3[1] = 255 - cmd;
	NbrOfDataToTransfer3 = 2;
	TxCounter3 = 0;
	USART_ITConfig(USART3, USART_IT_TXE, ENABLE);
}


void USART3_IRQHandler (void) {
	static uint8_t cmd_mode = 0;
	static uint16_t cmd_counter = 0;
	static uint8_t cmd_buffer[200];
	static uint8_t last = 0;
	uint8_t RX;
	if(USART_GetITStatus(USART3, USART_IT_RXNE) != RESET) {
		RX = USART_ReceiveData(USART3);
		if (cmd_mode == 1) {
			if (RX == ACT_DSL_GET_FREQTABLE_REP) {
				cmd_mode = ACT_DSL_GET_FREQTABLE;
				cmd_counter = 0;
			} else if (RX == ACT_DSL_GET_INFO_REP) {
				cmd_mode = ACT_DSL_GET_INFO;
				cmd_counter = 0;
			} else if (RX == 31) {
				cmd_mode = 31;
				cmd_counter = 0;
			} else {
				cmd_mode = 0;
			}

		} else if (cmd_mode == 31) {
			if (cmd_counter == 8) {
				ControlWrite(50, ((last<<8) + RX - 11000) / 36);
				cmd_counter++;
			} else if (cmd_counter == 10) {
				ControlWrite(51, ((last<<8) + RX - 11000) / 36);
				cmd_counter++;
			} else if (cmd_counter == 14) {
				ControlWrite(52, ((last<<8) + RX - 11000) / 36);
				cmd_counter++;
			} else if (cmd_counter == 16) {
				ControlWrite(53, ((last<<8) + RX - 11000) / 36);
				cmd_counter++;
			} else if (cmd_counter < 26) {
				cmd_counter++;
			} else {
				cmd_mode = 0;
			}

		} else if (cmd_mode == ACT_DSL_GET_INFO) {
			if (cmd_counter == 0) {
				cmd_counter++;
			} else if (cmd_counter < 26) {
				cmd_buffer[cmd_counter++ - 1] = RX;
			} else {
				USART1_SEND((char *)cmd_buffer, 26);
				cmd_mode = 0;
			}
		} else if (cmd_mode == ACT_DSL_GET_FREQTABLE) {
			if (cmd_counter == 0) {
				cmd_counter++;
			} else if (cmd_counter < 48) {
				cmd_buffer[cmd_counter++ - 1] = RX;
			} else {
				USART1_SEND((char *)cmd_buffer, 48);
				cmd_mode = 0;
			}
		} else {
			if (RX == 255 && last == 255) {
				cmd_mode = 1;
			}
		}


/*
		if (counter == 9) {
			ControlWrite(20, ((last<<8) + RX - 11000) / 40);
		} else if (counter == 11) {
			ControlWrite(21, ((last<<8) + RX - 11000) / 40);
		} else if (counter == 15) {
			ControlWrite(22, ((last<<8) + RX - 11000) / 40);
		} else if (counter == 17) {
			ControlWrite(23, ((last<<8) + RX - 11000) / 40);
		}

		if (counter < 250) {
			counter++;
		}

		if (RX == 255 && last == 255) {

			counter = 0;
		}
*/

		last = RX;


	}
	if(USART_GetITStatus(USART3, USART_IT_TXE) != RESET) {   
		USART_SendData(USART3, TxBuffer3[TxCounter3++]);
		if(TxCounter3 == NbrOfDataToTransfer3) {
			USART_ITConfig(USART3, USART_IT_TXE, DISABLE);
			NbrOfDataToTransfer3 = 0;
			TxCounter3 = 0;
		}    
	}
}

#endif
