
#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <rc_adc.h>
#include <spi_sw.h>
#include <font.h>
#include <icons.h>
#include <glcd.h>
#include <lcd.h>
#include <misc.h>

#ifdef SUPPORT_LCD

uint8_t Framebuffer[((LCD_W * LCD_H) / 8)];
uint8_t *font[] = {(uint8_t *)&au8FontSystem3x6, (uint8_t *)&au8FontSystem5x8, (uint8_t *)&au8FontSystem7x8};
const uint8_t font_w[] = {3, 5, 7};
const uint8_t font_h[] = {6, 8, 8};
uint8_t font_num = 0;

void LCD_Font(uint8_t num) {
	font_num = num;
}

inline void LCD_Clear (void) {
	memset(&Framebuffer, 0, (LCD_W * LCD_H) / 8);
}

void LCD_Pixel (uint8_t x, uint8_t y, uint8_t data) {
	uint16_t index = ((y / 8) * LCD_W) + x;
	if (x < LCD_W && y < LCD_H) {
		uint8_t offset = y - ((y / 8) * 8);
		switch (data) {
			case LCD_WHITE:
				Framebuffer[index] |= (1<<offset);
			break;
			case LCD_BLACK:
				Framebuffer[index] &= ~(1<<offset);
			break;
			case LCD_XOR:
				if (Framebuffer[index] & (1<<offset)) {
					Framebuffer[index] &= ~(1<<offset);
				} else {
					Framebuffer[index] |= (1<<offset);
				}
			break;
		}
	}
}

uint8_t LCD_Write (uint8_t x, uint8_t y, uint8_t ch, uint8_t fg, uint8_t bg) {
	uint8_t n = 0;
	uint8_t n2 = 0;
	uint8_t ret = 0;
	uint8_t y_offset = 8 - font_h[font_num];
	if (x + font_w[font_num] < LCD_W) {
		if (ch >= 32) {
			ch -= 32;
			for (n = 0; n < font_w[font_num]; n++) {
				for (n2 = 0; n2 < font_h[font_num]; n2++) {
					if (font[font_num][ch * font_w[font_num] + n] & (1<<(n2 + y_offset))) {
						ret = n;
						LCD_Pixel(x + n, y + n2, fg);
					} else if (bg != LCD_TRANS) {
						LCD_Pixel(x + n, y + n2, bg);
					}
				}
			}
			if (bg != LCD_TRANS) {
				for (n2 = 0; n2 < font_h[font_num]; n2++) {
					LCD_Pixel(x + n, y + n2, bg);
				}
			}
		}
	}
	return ret;
}

uint8_t LCD_CheckWidth (char ch) {
	uint8_t n = 0;
	uint8_t ret = 0;
	for (n = 0; n < font_w[font_num]; n++) {
		if (font[font_num][(ch - 32) * font_w[font_num] + n] != 0) {
			ret = n;
		}
	}
	return ret;
}

uint8_t LCD_CheckStrWidth (char *text) {
	uint8_t n = 0;
	uint8_t w = 0;
	uint8_t ret = 0;
	while (*text) {
		w = 0;
		for (n = 0; n < font_w[font_num]; n++) {
			if (font[font_num][((*text) - 32) * font_w[font_num] + n] != 0) {
				w = n;
			}
		}
		*text++;
		ret += w + 2;
	}
	return ret;
}

uint8_t LCD_Print_Center (uint8_t y, char *text, uint8_t fg, uint8_t bg) {
	uint8_t x = 0;
	char *text_ptr = text;
	while (*text_ptr) {
		x += LCD_CheckWidth(*text_ptr++) + 2;
	}
	text_ptr = text;
	x = (LCD_W - x) / 2;
	while (*text_ptr) {
		x += LCD_Write(x, y, (*text_ptr++), fg, bg) + 2;
	}
	return x;
}

uint8_t LCD_Print_Right (uint8_t y, char *text, uint8_t fg, uint8_t bg) {
	uint8_t x = 0;
	char *text_ptr = text;
	while (*text_ptr) {
		x += LCD_CheckWidth(*text_ptr++) + 2;
	}
	text_ptr = text;
	x = LCD_W - x;
	while (*text_ptr) {
		x += LCD_Write(x, y, (*text_ptr++), fg, bg) + 2;
	}
	return x;
}

uint8_t LCD_PrintXY (uint8_t x, uint8_t y, char *text, uint8_t fg, uint8_t bg) {
	if (x == 255) {
		return LCD_Print_Center(y, text, fg, bg);
	} else if (x == 254) {
		return LCD_Print_Right(y, text, fg, bg);
	}
	while (*text) {
		x += LCD_Write(x, y, (*text++), fg, bg) + 2;
	}
	return x;
}

void LCD_Icon (uint8_t x, uint8_t y, uint8_t w, uint8_t h, char *icon_data, char color) {
	uint8_t x1 = 0;
	uint8_t y1 = 0;
	for (y1 = 0; y1 < h; y1++) {
		for (x1 = 0; x1 < w; x1++) {
			if (icon_data[(y1 * w) + x1] != 0) {
				LCD_Pixel(x + x1, y + y1, color);
			}
		}
	}
}

void LCD_Draw_Rec (uint16_t xstart, uint16_t ystart, uint16_t width, uint16_t height) {
	uint16_t x = 0;
	uint16_t y = 0;
	for (y = ystart; y <= ystart + height; y++) {
		LCD_Pixel(xstart, y, LCD_WHITE);
		LCD_Pixel(xstart + width, y, LCD_WHITE);
	}
	for (x = xstart; x <= xstart + width; x++) {
		LCD_Pixel(x, ystart, LCD_WHITE);
		LCD_Pixel(x, ystart + height, LCD_WHITE);
	}
}

void LCD_Draw_Box (uint16_t xstart, uint16_t ystart, uint16_t width, uint16_t height) {
	uint16_t x = 0;
	uint16_t y = 0;
	for (y = ystart; y < ystart + height; y++) {
		for (x = xstart; x < xstart + width; x++) {
			LCD_Pixel(x, y, LCD_WHITE);
		}
	}
}

void LCD_Draw_BoxInverted (uint16_t xstart, uint16_t ystart, uint16_t width, uint16_t height) {
	uint16_t x = 0;
	uint16_t y = 0;
	for (y = ystart; y < ystart + height; y++) {
		for (x = xstart; x < xstart + width; x++) {
			LCD_Pixel(x, y, LCD_BLACK);
		}
	}
}

void LCD_Bar (uint8_t pos_x, uint8_t pos_y, uint8_t w, uint8_t h, uint8_t percent, char *text) {
	uint8_t x = 0;
	uint8_t y = pos_y;
	char *text_ptr = text;

	LCD_Font(1);

	LCD_Draw_BoxInverted(pos_x, pos_y, w, h);
	LCD_Draw_Box(pos_x, pos_y, percent * w / 100, h);
	LCD_Draw_Rec(pos_x, pos_y, w, h);
	while (*text_ptr) {
		x += LCD_CheckWidth(*text_ptr++) + 2;
	}
	text_ptr = text;
	x = (w - x) / 2;
	while (*text_ptr) {
		x += LCD_Write(pos_x + x, y + 2, (*text_ptr++), LCD_XOR, LCD_TRANS) + 2;
	}
}

void LCD_Header (char *text) {
	uint8_t pos_x = 0;
	char tmp_str[10];

	LCD_Font(2);
	LCD_Print_Center(0, text, LCD_XOR, LCD_TRANS);

	LCD_Font(1);
	int2str_new((char *)tmp_str, (SystemTimer_time[0] / 1000) / 60, 1, 0, 1);
	pos_x = LCD_PrintXY(pos_x, 0, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	pos_x = LCD_PrintXY(pos_x, 0, (char *)":", LCD_XOR, LCD_TRANS);
	int2str_new((char *)tmp_str, (uint16_t)((SystemTimer_time[0] / 1000) - ((SystemTimer_time[0] / 1000) / 60 * 60)), 1, 0, 1);
	pos_x = LCD_PrintXY(pos_x, 0, (char *)tmp_str, LCD_XOR, LCD_TRANS);

	uint16_t volt = GetADCChanel(ADC_CHANNELS_MAX - 2) * 1540 / 4096;
	int2str_new((char *)tmp_str, (int16_t)volt, 0, 0, 1);
	pos_x = LCD_CheckStrWidth((char *)tmp_str);
	pos_x += LCD_CheckStrWidth((char *)"V");
	pos_x = LCD_PrintXY(LCD_W - pos_x, 0, (char *)tmp_str, LCD_XOR, LCD_TRANS);
	pos_x = LCD_PrintXY(pos_x, 0, (char *)"V", LCD_XOR, LCD_TRANS);

	LCD_Font(1);
}

void LCD_Draw_Line(uint8_t px1, uint8_t py1, uint8_t px2, uint8_t py2, uint8_t color) {
	int16_t x, y, x1, x2, y1, y2, m, ofs;
	uint8_t counter;
	x1 = (uint16_t)px1;
	x2 = (uint16_t)px2;
	y1 = (uint16_t)py1;
	y2 = (uint16_t)py2;
	if (x2 < x1) {
		x = x1; x1 = x2; x2 = x;
		y = y1; y1 = y2; y2 = y;
	}
	if (y2 == y1) {
		for (counter = x1;counter <= x2;counter++) {
			LCD_Pixel((int8_t)counter, (int8_t)y1, color);
		}
	} else{
		if (x2 == x1) {
			if (y2 < y1) {
				x = x1;
				x1 = x2;
				x2 = x;
				y = y1;
				y1 = y2;
				y2 = y;
			}
			for (counter = y1; counter<= y2; counter++) {
				LCD_Pixel((uint8_t)x1, (uint8_t)counter, color);
			}
		} else {
			m = (127 * (y2 - y1)) / (x2 - x1);
			if ((m < 127) && (m > - 127)) {
				m = (500 * (y2 - y1)) / (x2 - x1);
				if (m >= 0) {
					ofs = 249;
				} else {
					ofs = -249;
				}
				for (counter = 0; counter <= (x2 - x1); counter++) {
					y = (counter*m + ofs) / 500;
					LCD_Pixel((uint8_t)counter + (uint8_t)x1, (uint8_t)y + y1, color);
				}
			} else {
				if ((m >= 127) || (m <= - 127)) {
					if (y2 < y1) {
						x = x1; x1 = x2; x2 = x;
						y = y1; y1 = y2; y2 = y;
					}
					m = (255*(y2 - y1)) / (x2 - x1);
					for (counter = 0; counter <= (y2-y1); counter++) {
						x = ((counter*255) + 127) / m;
						LCD_Pixel((uint8_t)(x+x1), (uint8_t)counter+(uint8_t)y1, color);
					}
				}
			}
		}
	}
}

/*
void LCD_Circle(uint8_t x0, uint8_t y0, uint8_t radius, uint8_t color) {
	int8_t f = 1 - radius;
	int8_t ddF_x = 0;
	int8_t ddF_y = -2 * radius;
	int8_t x = 0;
	int8_t y = radius;
	LCD_Pixel(x0, y0 + radius, color);
	LCD_Pixel(x0, y0 - radius, color);
	LCD_Pixel(x0 + radius, y0, color);
	LCD_Pixel(x0 - radius, y0, color);
	while(x < y) {
		if(f >= 0) {
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x + 1;
		LCD_Pixel(x0 + x, y0 + y, color);
		LCD_Pixel(x0 - x, y0 + y, color);
		LCD_Pixel(x0 + x, y0 - y, color);
		LCD_Pixel(x0 - x, y0 - y, color);
		LCD_Pixel(x0 + y, y0 + x, color);
		LCD_Pixel(x0 - y, y0 + x, color);
		LCD_Pixel(x0 + y, y0 - x, color);
		LCD_Pixel(x0 - y, y0 - x, color);
	}
}

void LCD_CircleFilled(uint8_t x0, uint8_t y0, uint8_t radius, uint8_t color) {
	int8_t f = 1 - radius;
	int8_t ddF_x = 0;
	int8_t ddF_y = -2 * radius;
	int8_t x = 0;
	int8_t y = radius;
	LCD_Draw_Line(x0,y0 - radius,x0,y0 + radius, color);
	LCD_Draw_Line(x0 - radius, y0, x0 + radius, y0, color);
	while(x < y) {
		if(f >= 0) {
			y--;
			ddF_y += 2;
			f += ddF_y;
		}
		x++;
		ddF_x += 2;
		f += ddF_x + 1;
		LCD_Draw_Line(x0 + x, y0 - y, x0 + x, y0 + y, color);
		LCD_Draw_Line(x0 + y, y0 - x, x0 + y, y0 + x, color);
		LCD_Draw_Line(x0 - x, y0 - y, x0 - x, y0 + y, color);
		LCD_Draw_Line(x0 - y, y0 - x, x0 - y, y0 + x, color);
	}
}
*/

#endif


