/*
Copyright (C) 2008-2009  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <rc_shiftreg.h>
#include <rc_ppm-encoder.h>
#include <rc_adc.h>
#include <rc_control.h>
#include <spi_sw.h>
#include <sd.h>
#include <glcd.h>
#include <lcd.h>
#include <sound.h>

uint16_t lcd_msg_time = 0;
uint8_t lcd_count = 0;
volatile char display_message_str[2][22];
volatile uint8_t display_control = 0;

void display_message (char *message1, char *message2, uint8_t time) {
	strcpy((char *)display_message_str[0], message1);
	strcpy((char *)display_message_str[1], message2);
	lcd_msg_time = time;
}

void BootMSG (uint8_t percent, char *text) {
#ifdef SUPPORT_LCD
	LCD_Bar(10, (LCD_H - 11), LCD_W - 20, 10, percent, text);
	LCD_Update();
#endif
	if (percent == 255) {
		sound(1500, 100);
		MotionTimer = 0; while (MotionTimer < 250) {}
	}
}


#ifdef SUPPORT_LCD
#ifdef LCD_WELLCOME_MSG

// 86, 29
static char logo_rcos[] = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,
	1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,
	1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,
	1,1,1,1,1,1,1,1,1,1,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,1,1,1,1,1,1,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	1,1,1,1,1,0,0,1,1,1,1,1,1,1,1,1,
	1,1,1,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,1,
	1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,0,
	0,0,1,1,1,1,1,0,0,1,1,1,1,1,1,1,
	1,1,1,1,1,0,
	1,1,1,1,1,0,0,1,1,1,1,1,1,1,1,1,
	1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,1,1,1,1,1,1,1,
	1,1,1,1,1,1,
	1,1,1,1,1,0,0,1,1,1,1,1,1,1,1,1,
	1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,1,1,1,1,1,1,1,
	1,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1,
	1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1,
	1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1,
	1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,1,
	1,1,1,1,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,0,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,0,0,0,
	0,0,0,0,1,1,1,1,1,1,0,0,1,1,1,1,
	1,0,0,0,0,0,0,0,0,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,0,0,0,0,0,0,0,0,0,
	0,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,
	1,1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,
	1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,
	1,1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,
	1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,
	0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,1,
	1,1,1,1,1,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,0,0,1,1,1,1,1,0,0,1,1,1,
	1,1,1,1,1,1,1,1,1,0,0,0,0,1,1,1,
	1,1,1,1,1,1,1,1,1,0,0,1,1,1,1,1,
	0,0,0,1,1,1,1,1,1,1,1,1,1,1,1,1,
	1,1,1,1,1,0
};

#ifdef RCOS_MAINBOARD_V1_0
#define LOGO_VERSION_W	20
#define LOGO_VERSION_H	9
static char logo_version[] = {
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,
	0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,0,
	0,0,0,0,
	1,1,1,1,1,1,0,1,1,1,1,1,1,0,1,1,
	1,1,1,1,
	1,1,0,0,1,1,0,1,1,0,0,1,1,0,1,1,
	0,0,1,1,
	1,1,0,0,1,1,0,1,1,0,0,1,1,0,1,1,
	0,0,1,1,
	1,1,0,0,1,1,0,1,1,0,0,1,1,0,1,1,
	0,0,1,1,
	1,1,0,0,1,1,0,1,1,0,0,1,1,0,1,1,
	1,1,1,1,
	1,1,0,0,1,1,0,1,1,0,0,1,1,0,1,1,
	0,0,0,0,
	1,1,1,1,1,1,0,1,1,0,0,1,1,0,1,1,
	1,1,1,1
};
#endif

#ifdef RCOS_MAINBOARD_FC16
#define LOGO_VERSION_W	17
#define LOGO_VERSION_H	9
static char logo_version[] = {
	1,1,1,0,1,1,0,0,1,1,0,0,0,0,0,0,
	0,
	0,1,1,0,0,0,0,0,1,1,0,0,0,0,0,0,
	0,
	0,1,1,0,1,1,1,1,1,1,1,1,1,1,1,1,
	1,
	0,1,1,0,1,1,0,0,1,1,0,1,1,0,0,1,
	1,
	0,1,1,0,1,1,0,0,1,1,0,1,1,0,0,1,
	1,
	0,1,1,0,1,1,0,0,1,1,0,1,1,0,0,1,
	1,
	0,1,1,0,1,1,0,0,1,1,0,1,1,1,1,1,
	1,
	0,1,1,0,1,1,0,0,1,1,0,1,1,0,0,0,
	0,
	0,1,1,0,1,1,1,0,1,1,1,1,1,1,1,1,
	1
};
#endif
#endif
#endif

