
#include <config.h>
#include <stdint.h>
#ifdef PLATFORM_STM32
#include <stm32f10x_lib.h>
#include <stm32f10x_conf.h>
#endif
#ifdef PLATFORM_SDL
#include <stdio.h>
#include <SDL.h>
#endif

#ifndef RC_H
#define RC_H

#define NAME_MAX				14
#define MODEL_NAME_MAX			32
#define TIMER_MAX				4

#define CONTROL_ON				253
#define CONTROL_OFF				254
#define CONTROL_NONE			255
#define CONTROL_MAX				255

#define LOGIC_MAX				32

#define INPUT_ADC_MAX			16

#define DUALRATE_MAX			16
#define MOTION_MAX				16
#define EXPO_MAX				16
#define ATV_MAX					16
#define LIMITER_MAX				16

#define MIXER_MAX				64
#define MIXER_POINTS_MAX		7

#define OUTPUT_MAX				64

#define PPM1_CHANNELS_MAX		16
#define PPM2_CHANNELS_MAX		16

#define PPM1_MENCODER_MAX			2
#define PPM1_MENCODER_CHANNEL_MAX	16

#define ACT_DSL_GET_INFO			224
#define ACT_DSL_GET_INFO_REP		225
#define ACT_DSL_GET_FREQTABLE		226
#define ACT_DSL_GET_FREQTABLE_REP	227
#define ACT_DSL_GET_SERVO			230
#define ACT_DSL_GET_SERVO_REP		231
#define ACT_DSL_GET_SETUP			234
#define ACT_DSL_GET_SETUP_REP		235


enum {
	CMD_NONE,
	CMD_UP,
	CMD_ENTER,
	CMD_DOWN,
	CMD_BACK,
};

enum {
	FE_FLAG_INPUT_REVERSE,
	FE_FLAG_OUTPUT_REVERSE,
	FE_FLAG_CONTROL_REVERSE,
	FE_FLAG_CONTROL_TYPE1,
	FE_FLAG_CONTROL_TYPE2,
	FE_FLAG_CONTROL_TYPE3,
};

enum {
	FE_CONTROL_TYPE_1SWITCH,
	FE_CONTROL_TYPE_2SWITCH,
	FE_CONTROL_TYPE_1PULSE,
	FE_CONTROL_TYPE_2PULSE,
	FE_CONTROL_TYPE_ENCODER,
	FE_CONTROL_TYPE_SHIFTREG,
	FE_CONTROL_TYPE_EXT1,
	FE_CONTROL_TYPE_EXT2,
};

enum {
	MOTION_TYPE_INPUT_SPEED,
	MOTION_TYPE_INPUT_SET,
	MOTION_TYPE_OUTPUT_SPEED,
	MOTION_TYPE_OUTPUT_SET,
};

enum {
	FE_CMD_NONE,
	FE_CMD_CONTROL_GET,
	FE_CMD_CONTROL_SET,
	FE_CMD_CONTROL_VALUE_GET,
	FE_CMD_CONTROL_VALUE_SET,
	FE_CMD_MIXER_GET,
	FE_CMD_MIXER_SET,
	FE_CMD_OUTPUT_GET,
	FE_CMD_OUTPUT_SET,
	FE_CMD_DUALRATE_GET,
	FE_CMD_DUALRATE_SET,
	FE_CMD_EXPO_GET,
	FE_CMD_EXPO_SET,
	FE_CMD_MOTION_GET,
	FE_CMD_MOTION_SET,
	FE_CMD_LIMITER_GET,
	FE_CMD_LIMITER_SET,
	FE_CMD_ATV_GET,
	FE_CMD_ATV_SET,
	FE_CMD_ACT_FREQ_GET,
	FE_CMD_ACT_INFO_GET,
	FE_CMD_PPMENCODER1_MENCODER_GET,
	FE_CMD_PPMENCODER1_MENCODER_SET,
	FE_CMD_PPMENCODER1_GET,
	FE_CMD_PPMENCODER1_SET,
	FE_CMD_PPMENCODER2_GET,
	FE_CMD_PPMENCODER2_SET,
	FE_CMD_SERVO_TEST,
	FE_CMD_LAST,
};

enum {
	LOGIG_TEST_NONE, // None
	LOGIG_TEST_AND_EQ, // =
	LOGIG_TEST_AND_NE, // !=
	LOGIG_TEST_AND_LT, // <
	LOGIG_TEST_AND_GT, // >
	LOGIG_TEST_AND_LE, // <=
	LOGIG_TEST_AND_GE, // >=
	LOGIG_TEST_OR_EQ, // =
	LOGIG_TEST_OR_NE, // !=
	LOGIG_TEST_OR_LT, // <
	LOGIG_TEST_OR_GT, // >
	LOGIG_TEST_OR_LE, // <=
	LOGIG_TEST_OR_GE, // >=
};


// Rc_Calibrate: Stick-Kalibrierung
extern int16_t Rc_Calibrate_center[4];						// Stick-Kalibrierung (Mittelpunkt)
extern int16_t Rc_Calibrate_min[4];							// Stick-Kalibrierung (Minimal-Ausschlag)
extern int16_t Rc_Calibrate_max[4];							// Stick-Kalibrierung (Maximal-Ausschlag)

// Rc_Model: Modell-Info's
extern uint8_t Rc_Model_icon;								// Modell-Icon-Num
extern uint8_t Rc_Model_name[MODEL_NAME_MAX + 1];			// Modell-Name
extern uint8_t Rc_Model_backcolor;							// LCD-Hintergrund-Farbe
extern uint8_t Rc_Model_forecolor;							// LCD-Vordergrund-Farbe

// SystemTimer: Modell-Timer mit je 3 Alarm-Zeiten
extern uint8_t SystemTimer_name[TIMER_MAX][NAME_MAX];		// Timer-Name
extern uint8_t SystemTimer_control[TIMER_MAX];				// Geber zum Schalten des Timers
extern uint32_t SystemTimer_alarm1[TIMER_MAX];				// Alarm 1
extern uint32_t SystemTimer_alarm2[TIMER_MAX];				// Alarm 2
extern uint32_t SystemTimer_alarm3[TIMER_MAX];				// Alarm 3
extern volatile uint8_t SystemTimer_stat[TIMER_MAX];		// Status des Timer
extern volatile uint32_t SystemTimer_time[TIMER_MAX];		// Timer-Counter

// Rc_Control: Geber-Setup
extern uint8_t Rc_Control_name[CONTROL_MAX][NAME_MAX];		// Geber-Name
extern int8_t Rc_Control_value[CONTROL_MAX];				// Geber-Wert
extern int8_t Rc_Control_reverse[CONTROL_MAX];				// Geber-Reverse
extern int16_t Rc_Control_buffer[CONTROL_MAX];				// Geber-Werte-Puffer

// Rc_LogicControl: Logische-Geber
extern uint8_t Rc_LogicControl_target[LOGIC_MAX];			// Logische-Geber (Ziel-Geber)
extern uint8_t Rc_LogicControl_control[LOGIC_MAX];			// Logische-Geber (Test-Geber)
extern uint8_t Rc_LogicControl_test[LOGIC_MAX];				// Logische-Geber (Test-Funktion)
extern uint8_t Rc_LogicControl_value[LOGIC_MAX];			// Logische-Geber (Vergleichs-Wert)

// Rc_Motion: Geschwindigkeits-Regulierung und Soft-Start-Stop
extern uint8_t Rc_Motion_type[MOTION_MAX];;					// Art der Motion-Funktion (Speed-IN, Speed-Out, Motion-IN, Motion-Out)
extern uint8_t Rc_Motion_switch[MOTION_MAX];				// Geber zum An/Aus Schalten der Funktion
extern uint8_t Rc_Motion_up_control[MOTION_MAX];			// Geber zum Regeln der Miximal-Geschwindigkeit (1. Richtung)
extern uint8_t Rc_Motion_up_value[MOTION_MAX];				// Miximal-Geschwindigkeit (schritte/systick) (1. Richtung)
extern uint8_t Rc_Motion_down_control[MOTION_MAX];			// Geber zum Regeln der Miximal-Geschwindigkeit (2. Richtung)
extern uint8_t Rc_Motion_down_value[MOTION_MAX];			// Miximal-Geschwindigkeit (schritte/systick) (2. Richtung)
extern uint8_t Rc_Motion_softstart_speed[MOTION_MAX];		// Softstart-Funktion (0-100%)
extern uint8_t Rc_Motion_channel[MOTION_MAX];				// Kanal auf dem diese Funktion wirken soll
extern int16_t Rc_Motion_buffer[MOTION_MAX];
extern int16_t Rc_Motion_softstart_buffer[MOTION_MAX];

// Rc_Expo: Expo-Funktion
extern uint8_t Rc_Expo_switch[EXPO_MAX];					// Geber zum An/Aus Schalten der Funktion
extern uint8_t Rc_Expo_gradient_control[EXPO_MAX];			// Geber zum Regeln der Steigung
extern uint8_t Rc_Expo_gradient_value[EXPO_MAX];			// Steigung der Expo-Funktion (+-0-100%)
extern uint8_t Rc_Expo_type[EXPO_MAX];						// Typ der Expo-Funktion (hoch2, hoch3)
extern uint8_t Rc_Expo_channel[EXPO_MAX];					// Kanal auf dem diese Funktion wirken soll

// Rc_Dualrate: Dualrate-Funktion
extern uint8_t Rc_Dualrate_switch[DUALRATE_MAX];			// Geber zum An/Aus Schalten der Funktion
extern uint8_t Rc_Dualrate_limit_control[DUALRATE_MAX];		// Geber zum Regeln des Maximal-Ausschlags
extern uint8_t Rc_Dualrate_limit_value[DUALRATE_MAX];		// Wert des Maximal-Ausschlags (0-100%)
extern uint8_t Rc_Dualrate_channel[DUALRATE_MAX];			// Kanal auf dem diese Funktion wirken soll

// Rc_Mixer: 7-Punkt-Mixer
extern uint8_t Rc_Mixer_switch[MIXER_MAX];					// Geber zum An/Aus Schalten des Mixers
extern uint8_t Rc_Mixer_percent_control[MIXER_MAX];			// Geber zum Regeln des Mixer-Anteils
extern uint8_t Rc_Mixer_percent_value[MIXER_MAX];			// Wert des Mixer-Anteils (0-100%)
extern uint8_t Rc_Mixer_point[MIXER_MAX][MIXER_POINTS_MAX][2];		// 7-Punkt-Kurve (1xy-7xy, 0-100%)
extern uint8_t Rc_Mixer_channel_in[MIXER_MAX];				// Eingans-Kanal des Mixer's
extern uint8_t Rc_Mixer_channel_out[MIXER_MAX];				// Ausgangs-Kanal des Mixer's

// Rc_Atv: Nach beiden Richtungen getrennt einstellbare Servo-Wegverk&uuml;rzung
extern uint8_t Rc_Atv_switch[ATV_MAX];						// Geber zum An/Aus Schalten der Funktion
extern uint8_t Rc_Atv_up_control[ATV_MAX];					// Geber zum Regeln des Miximal-Ausschlags (1. Richtung)
extern uint8_t Rc_Atv_up_value[ATV_MAX];					// Miximal-Ausschlag (schritte/systick) (1. Richtung)
extern uint8_t Rc_Atv_down_control[ATV_MAX];				// Geber zum Regeln des Miximal-Ausschlags (2. Richtung)
extern uint8_t Rc_Atv_down_value[ATV_MAX];					// Miximal-Ausschlag (schritte/systick) (2. Richtung)
extern uint8_t Rc_Atv_channel[ATV_MAX];						// Kanal auf dem diese Funktion wirken soll

// Rc_Limiter: Nach beiden Richtungen getrennt einstellbarer Servo-Anschlag
extern uint8_t Rc_Limiter_switch[LIMITER_MAX];					// Geber zum An/Aus Schalten der Funktion
extern uint8_t Rc_Limiter_up_control[LIMITER_MAX];				// Geber zum Regeln des Miximal-Ausschlags (1. Richtung)
extern uint8_t Rc_Limiter_up_value[LIMITER_MAX];				// Miximal-Ausschlag (schritte/systick) (1. Richtung)
extern uint8_t Rc_Limiter_down_control[LIMITER_MAX];			// Geber zum Regeln des Miximal-Ausschlags (2. Richtung)
extern uint8_t Rc_Limiter_down_value[LIMITER_MAX];				// Miximal-Ausschlag (schritte/systick) (2. Richtung)
extern uint8_t Rc_Limiter_channel[LIMITER_MAX];					// Kanal auf dem diese Funktion wirken soll

// Rc_Output: Ausgangs-Kan&auml;le / Servo-Setup
extern uint8_t Rc_Output_name[OUTPUT_MAX][NAME_MAX];			// Geber zum Regeln der Trimmung
extern uint8_t Rc_Output_center_control[OUTPUT_MAX];			// Geber zum Regeln der Trimmung
extern int8_t Rc_Output_center_value[OUTPUT_MAX];				// Default-Wert der Trimmung (+-0-100%)
extern int8_t Rc_Output_reverse[OUTPUT_MAX];					// Servo-Reverse
extern int16_t Rc_Output_value[OUTPUT_MAX];						// Aktueller-Ausgangs-Wert

// Rc_PPMEncoder1: 1. Variabler PPM-Encoder (inkl. Multiprop-Encoder)
extern volatile uint8_t Rc_PPMEncoder1_channels;				// Anzahl der zu sendenden Kan&auml;le
extern volatile uint16_t Rc_PPMEncoder1_shift;					// L&auml;nge des Shift-Pulses
extern volatile uint16_t Rc_PPMEncoder1_sync;					// L&auml;nge des Sync-Pulses
extern volatile uint16_t Rc_PPMEncoder1_center;					// L&auml;nge des Center-Pulses
extern volatile uint16_t Rc_PPMEncoder1_range;					// Maximale Differenz zum Center-Pulses
extern volatile uint8_t Rc_PPMEncoder1_invert;					// Invertieren des PPM-Impuses
extern volatile uint8_t Rc_PPMEncoder1_power;					// An/Aus-Schalten des HF-Moduls
extern volatile uint8_t Rc_PPMEncoder1_frame_sync;				// Frame-Sync-Flag (intern)
extern volatile int8_t Rc_PPMEncoder1_Mencoder_type[PPM1_MENCODER_MAX];		// Multiprop-Encoder auf PPM1
extern volatile uint8_t Rc_PPMEncoder1_Mencoder_max[PPM1_MENCODER_MAX];		// Anzahl der Multiporp-Encoder
extern volatile uint8_t Rc_PPMEncoder1_Mencoder_channel[PPM1_MENCODER_MAX];	// Übertragungs-Kanal des Encoders
extern volatile int16_t Rc_PPMEncoder1_Mencoder_values[PPM1_MENCODER_MAX][PPM1_MENCODER_CHANNEL_MAX];	// Multiporp-Werte (intern)
extern volatile int16_t Rc_PPMEncoder1_values[PPM1_CHANNELS_MAX];		// PPM-Ausgangs-Werte (intern)

// Rc_PPMEncoder2: 2. Variabler PPM-Encoder
extern volatile uint8_t Rc_PPMEncoder2_channels;				// Anzahl der zu sendenden Kan&auml;le
extern volatile uint16_t Rc_PPMEncoder2_shift;					// L&auml;nge des Shift-Pulses
extern volatile uint16_t Rc_PPMEncoder2_sync;					// L&auml;nge des Sync-Pulses
extern volatile uint16_t Rc_PPMEncoder2_center;					// L&auml;nge des Center-Pulses
extern volatile uint16_t Rc_PPMEncoder2_range;					// Maximale Differenz zum Center-Pulses
extern volatile uint8_t Rc_PPMEncoder2_invert;					// Invertieren des PPM-Impuses
extern volatile uint8_t Rc_PPMEncoder2_power;					// An/Aus-Schalten des HF-Moduls
extern volatile uint8_t Rc_PPMEncoder2_frame_sync;				// Frame-Sync-Flag (intern)
extern volatile int16_t Rc_PPMEncoder2_values[PPM2_CHANNELS_MAX];		// PPM-Ausgangs-Werte (intern)

extern uint8_t ModelID;
extern uint8_t Last_Control;
extern uint8_t Servo_Testflag;
extern volatile uint8_t MotionTimer;
extern volatile uint8_t SystemTimer;
extern uint16_t FeCmdLen[FE_CMD_LAST];


extern void Rc_Stage_Input (void);
extern void Rc_Stage_Mixer (void);
extern void Rc_Stage_Output (void);

extern void MotionTimer_reset (void);
extern int16_t AdcRead (int8_t id);
extern int8_t ControlSet(uint8_t control, int8_t value);
extern int8_t ControlRead (uint8_t id);
extern uint16_t Rc_Cmd_LogicControl_Get (uint8_t *buffer);
extern void Rc_Cmd_LogicControl_Set (uint8_t *buffer);
extern void Rc_REncoderDecode_byte (uint8_t byte, uint8_t bit_a, uint8_t bit_b, uint8_t bit_sw, uint8_t control_id);
extern uint16_t Rc_Cmd_PPMEncoder1_Mencoder_Get (uint8_t *buffer);
extern void Rc_Cmd_PPMEncoder1_Mencoder_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_PPMEncoder1_Get (uint8_t *buffer);
extern void Rc_Cmd_PPMEncoder1_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_PPMEncoder2_Get (uint8_t *buffer);
extern void Rc_Cmd_PPMEncoder2_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Input_Get (uint8_t *buffer);
extern void Rc_Cmd_Input_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Output_Get (uint8_t *buffer);
extern void Rc_Cmd_Output_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Expo_Get (uint8_t *buffer);
extern void Rc_Cmd_Expo_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Dualrate_Get (uint8_t *buffer);
extern void Rc_Cmd_Dualrate_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Limiter_Get (uint8_t *buffer);
extern void Rc_Cmd_Limiter_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Atv_Get (uint8_t *buffer);
extern void Rc_Cmd_Atv_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Motion_Get (uint8_t *buffer);
extern void Rc_Cmd_Motion_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Mixer_Get (uint8_t *buffer);
extern void Rc_Cmd_Mixer_Set (uint8_t *buffer);
extern void Rc_Cmd_Control_Set (uint8_t *buffer);
extern uint16_t Rc_Cmd_Control_Get (uint8_t *buffer);
extern void Rc_Cmd_Init (void);
extern int8_t ControlSet(uint8_t control, int8_t value);
extern void Act_Cmd_Send(uint8_t cmd);

extern void delay_ms (uint32_t time);

#endif
