/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include "rc.h"
#include "rc_adc.h"
#include "rc_control.h"



uint16_t Rc_Dualrate_Do (int16_t value, int8_t limit) {
	int16_t new_value = value;
	if (limit > 0) {
		new_value = value * limit / 100;
	}
	return new_value;
}


uint16_t Rc_Expo_Do (int16_t value, int8_t gradient) {
	int16_t new_value = value;
	if (gradient > 0) {
		float val = (float)value / 1000.0;
		new_value = (val - (val - (val * val * val)) * ((float)gradient / 100.0)) * 1000.0;
	}
	return new_value;
}


void Rc_Stage_Input (void) {
	uint16_t num;
	int16_t new_value;
	int16_t new_value2;

	Rc_Control_value[CONTROL_OFF] = 0;
	Rc_Control_value[CONTROL_ON] = 100;

	for (num = 0; num < CONTROL_MAX; num++) {
		if (num < INPUT_ADC_MAX) {
			Rc_Control_buffer[num] = Rc_AdcRead(num);
		} else {
			Rc_Control_buffer[num] = ControlRead(num) * 8;
		}
		if (Rc_Control_reverse[num] == 1) {
			Rc_Control_buffer[num] *= -1;
		}
	}

	for (num = 0; num < DUALRATE_MAX; num++) {
		if (Rc_Dualrate_channel[num] < CONTROL_MAX && (Rc_Dualrate_switch[num] == CONTROL_ON || ControlRead(Rc_Dualrate_switch[num]) > 10)) {
			new_value = ControlSet(Rc_Dualrate_limit_control[num], Rc_Dualrate_limit_value[num]);
			Rc_Control_buffer[Rc_Dualrate_channel[num]] = Rc_Dualrate_Do(Rc_Control_buffer[Rc_Dualrate_channel[num]], new_value);
		}
	}

	for (num = 0; num < EXPO_MAX; num++) {
		if (Rc_Expo_channel[num] < CONTROL_MAX && (Rc_Expo_switch[num] == CONTROL_ON || ControlRead(Rc_Expo_switch[num]) > 10)) {
			new_value = ControlSet(Rc_Expo_gradient_control[num], Rc_Expo_gradient_value[num]);
			Rc_Control_buffer[Rc_Expo_channel[num]] = Rc_Expo_Do(Rc_Control_buffer[Rc_Expo_channel[num]], new_value);
		}
	}

	if (MotionTimer >= 10) {
		MotionTimer_reset();
		MotionTimer = 0;
		for (num = 0; num < MOTION_MAX; num++) {
			if (Rc_Motion_channel[num] < CONTROL_MAX && (Rc_Motion_switch[num] == CONTROL_ON || ControlRead(Rc_Motion_switch[num]) > 10)) {
				if (Rc_Motion_type[num] == MOTION_TYPE_INPUT_SPEED) {
					new_value = ControlSet(Rc_Motion_up_control[num], Rc_Motion_up_value[num]);
					new_value2 = ControlSet(Rc_Motion_down_control[num], Rc_Motion_down_value[num]);
					if (new_value < 0) {
						new_value *= -1;
					}
					if (new_value2 < 0) {
						new_value2 *= -1;
					}
					if (Rc_Control_buffer[Rc_Motion_channel[num]] > Rc_Motion_buffer[num]) {
						Rc_Motion_buffer[num] += new_value;
						if (Rc_Motion_up_control[num] == CONTROL_OFF || Rc_Motion_buffer[num] > Rc_Control_buffer[Rc_Motion_channel[num]]) {
							Rc_Motion_buffer[num] = Rc_Control_buffer[Rc_Motion_channel[num]];
						}
					} else if (Rc_Control_buffer[Rc_Motion_channel[num]] < Rc_Motion_buffer[num]) {
						Rc_Motion_buffer[num] -= new_value2;
						if (Rc_Motion_down_control[num] == CONTROL_OFF || Rc_Motion_buffer[num] < Rc_Control_buffer[Rc_Motion_channel[num]]) {
							Rc_Motion_buffer[num] = Rc_Control_buffer[Rc_Motion_channel[num]];
						}
					}
					Rc_Control_buffer[Rc_Motion_channel[num]] = Rc_Motion_buffer[num];
				} else if (Rc_Motion_type[num] == MOTION_TYPE_INPUT_SET) {
					new_value = ControlSet(Rc_Motion_up_control[num], Rc_Motion_up_value[num]);
					new_value2 = ControlSet(Rc_Motion_down_control[num], Rc_Motion_down_value[num]);
					if (new_value < 0) {
						new_value *= -1;
					}
					if (new_value2 > 0) {
						new_value2 *= -1;
					}

Rc_Motion_softstart_speed[num] = 1;

					if (Rc_Control_buffer[Rc_Motion_channel[num]] > 100 && Rc_Motion_buffer[num] < 1500) {
						if (Rc_Motion_softstart_speed[num] > 0) {
							if (Rc_Motion_softstart_buffer[num] < new_value) {
								Rc_Motion_softstart_buffer[num]++;
							}
						} else {
							Rc_Motion_softstart_buffer[num] = new_value;
						}
					} else if (Rc_Control_buffer[Rc_Motion_channel[num]] < -100 && Rc_Motion_buffer[num] > -1500) {
						if (Rc_Motion_softstart_speed[num] > 0) {
							if (Rc_Motion_softstart_buffer[num] < new_value2) {
								Rc_Motion_softstart_buffer[num]--;
							}
						} else {
							Rc_Motion_softstart_buffer[num] = new_value2;
						}
					} else {
						if (Rc_Motion_softstart_speed[num] > 0) {
							if (Rc_Motion_softstart_buffer[num] > 0) {
								Rc_Motion_softstart_buffer[num]--;
							} else if (Rc_Motion_softstart_buffer[num] < 0) {
								Rc_Motion_softstart_buffer[num]++;
							}
						} else {
							Rc_Motion_softstart_buffer[num] = 0;
						}
					}

					Rc_Motion_buffer[num] += Rc_Motion_softstart_buffer[num];

					if (Rc_Motion_buffer[num] < -2000) {
						Rc_Motion_buffer[num] = -2000;
					} else if (Rc_Motion_buffer[num] > 2000) {
						Rc_Motion_buffer[num] = 2000;
					}

					Rc_Control_buffer[Rc_Motion_channel[num]] = Rc_Motion_buffer[num];
				}
			}
		}
	} else {
		for (num = 0; num < MOTION_MAX; num++) {
			if (Rc_Motion_channel[num] < CONTROL_MAX && (Rc_Motion_switch[num] == CONTROL_ON || ControlRead(Rc_Motion_switch[num]) > 10)) {
				if (Rc_Motion_type[num] == MOTION_TYPE_INPUT_SPEED) {
					Rc_Control_buffer[Rc_Motion_channel[num]] = Rc_Motion_buffer[num];
				} else if (Rc_Motion_type[num] == MOTION_TYPE_INPUT_SET) {
					Rc_Control_buffer[Rc_Motion_channel[num]] = Rc_Motion_buffer[num];
				}
			}
		}
	}

}



