/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include "string.h"
#include "rc.h"
#include "rc_ppm-encoder.h"

volatile uint8_t SystemTimer = 0;
uint8_t Rc_Model_name[MODEL_NAME_MAX + 1];
uint8_t ModelID = 0;
uint8_t Rc_Model_icon = 0;
uint8_t Rc_Model_backcolor = 0x00;
uint8_t Rc_Model_forecolor = 0xFF;

uint8_t SystemTimer_name[TIMER_MAX][NAME_MAX];
uint32_t SystemTimer_alarm1[TIMER_MAX];
uint32_t SystemTimer_alarm2[TIMER_MAX];
uint32_t SystemTimer_alarm3[TIMER_MAX];
volatile uint8_t SystemTimer_stat[TIMER_MAX];
volatile uint32_t SystemTimer_time[TIMER_MAX];

uint8_t Last_Control = 0;
uint8_t SystemTimer_control[TIMER_MAX];
uint8_t Rc_Control_name[CONTROL_MAX][NAME_MAX];
int8_t  Rc_Control_value[CONTROL_MAX];
int8_t  Rc_Control_reverse[CONTROL_MAX];
int16_t  Rc_Control_buffer[CONTROL_MAX];

uint8_t Rc_LogicControl_target[LOGIC_MAX];
uint8_t Rc_LogicControl_control[LOGIC_MAX];
uint8_t Rc_LogicControl_test[LOGIC_MAX];
uint8_t Rc_LogicControl_value[LOGIC_MAX];

int16_t Rc_Calibrate_center[4];
int16_t Rc_Calibrate_min[4];
int16_t Rc_Calibrate_max[4];

uint8_t Rc_Expo_type[EXPO_MAX];
uint8_t Rc_Expo_channel[EXPO_MAX];
uint8_t Rc_Expo_switch[EXPO_MAX];
uint8_t Rc_Expo_gradient_control[EXPO_MAX];
uint8_t Rc_Expo_gradient_value[EXPO_MAX];

uint8_t Rc_Dualrate_channel[DUALRATE_MAX];
uint8_t Rc_Dualrate_switch[DUALRATE_MAX];
uint8_t Rc_Dualrate_limit_control[DUALRATE_MAX];
uint8_t Rc_Dualrate_limit_value[DUALRATE_MAX];

uint8_t Rc_Mixer_channel_in[MIXER_MAX];
uint8_t Rc_Mixer_channel_out[MIXER_MAX];
uint8_t Rc_Mixer_point[MIXER_MAX][MIXER_POINTS_MAX][2];
uint8_t Rc_Mixer_switch[MIXER_MAX];
uint8_t Rc_Mixer_percent_control[MIXER_MAX];
uint8_t Rc_Mixer_percent_value[MIXER_MAX];

uint8_t Rc_Limiter_channel[LIMITER_MAX];
uint8_t Rc_Limiter_switch[LIMITER_MAX];
uint8_t Rc_Limiter_up_control[LIMITER_MAX];
uint8_t Rc_Limiter_up_value[LIMITER_MAX];
uint8_t Rc_Limiter_down_control[LIMITER_MAX];
uint8_t Rc_Limiter_down_value[LIMITER_MAX];

uint8_t Rc_Motion_type[MOTION_MAX];
uint8_t Rc_Motion_channel[MOTION_MAX];
uint8_t Rc_Motion_softstart_speed[MOTION_MAX];
int16_t Rc_Motion_softstart_buffer[MOTION_MAX];
uint8_t Rc_Motion_switch[MOTION_MAX];
uint8_t Rc_Motion_up_control[MOTION_MAX];
uint8_t Rc_Motion_up_value[MOTION_MAX];
uint8_t Rc_Motion_down_control[MOTION_MAX];
uint8_t Rc_Motion_down_value[MOTION_MAX];
int16_t Rc_Motion_buffer[MOTION_MAX];

uint8_t Rc_Atv_channel[ATV_MAX];
uint8_t Rc_Atv_switch[ATV_MAX];
uint8_t Rc_Atv_up_control[ATV_MAX];
uint8_t Rc_Atv_up_value[ATV_MAX];
uint8_t Rc_Atv_down_control[ATV_MAX];
uint8_t Rc_Atv_down_value[ATV_MAX];

uint8_t Rc_Output_name[OUTPUT_MAX][NAME_MAX];
uint8_t Rc_Output_center_control[OUTPUT_MAX];
int8_t Rc_Output_center_value[OUTPUT_MAX];
int8_t Rc_Output_reverse[OUTPUT_MAX];
int16_t Rc_Output_value[OUTPUT_MAX];

volatile int8_t Rc_PPMEncoder1_Mencoder_type[PPM1_MENCODER_MAX];
volatile uint8_t Rc_PPMEncoder1_Mencoder_max[PPM1_MENCODER_MAX];
volatile uint8_t Rc_PPMEncoder1_Mencoder_channel[PPM1_MENCODER_MAX];
volatile int16_t Rc_PPMEncoder1_Mencoder_values[PPM1_MENCODER_MAX][PPM1_MENCODER_CHANNEL_MAX];

volatile int16_t Rc_PPMEncoder1_values[PPM1_CHANNELS_MAX];
volatile uint8_t Rc_PPMEncoder1_channels;
volatile uint16_t Rc_PPMEncoder1_shift;
volatile uint16_t Rc_PPMEncoder1_sync;
volatile uint16_t Rc_PPMEncoder1_center;
volatile uint16_t Rc_PPMEncoder1_range;
volatile uint8_t Rc_PPMEncoder1_invert;
volatile uint8_t Rc_PPMEncoder1_power;
volatile uint8_t Rc_PPMEncoder1_frame_sync;
volatile int16_t Rc_PPMEncoder2_values[PPM2_CHANNELS_MAX];
volatile uint8_t Rc_PPMEncoder2_channels;
volatile uint16_t Rc_PPMEncoder2_shift;
volatile uint16_t Rc_PPMEncoder2_sync;
volatile uint16_t Rc_PPMEncoder2_center;
volatile uint16_t Rc_PPMEncoder2_range;
volatile uint8_t Rc_PPMEncoder2_invert;
volatile uint8_t Rc_PPMEncoder2_power;
volatile uint8_t Rc_PPMEncoder2_frame_sync;

uint8_t Servo_Testflag;

volatile uint8_t MotionTimer;

uint16_t FeCmdLen[FE_CMD_LAST];


uint16_t Rc_Cmd_PPMEncoder1_Get (uint8_t *buffer) {
	uint16_t num = 0;
	buffer[num++] = Rc_PPMEncoder1_channels;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder1_shift, 2);
	num += 2;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder1_sync, 2);
	num += 2;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder1_center, 2);
	num += 2;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder1_range, 2);
	num += 2;
	buffer[num++] = Rc_PPMEncoder1_invert;
	buffer[num++] = Rc_PPMEncoder1_power;
	return num;
}

void Rc_Cmd_PPMEncoder1_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_PPMEncoder1_channels, buffer, 1);
	buffer++;
	memcpy((char *)&Rc_PPMEncoder1_shift, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder1_sync, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder1_center, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder1_range, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder1_invert, buffer, 1);
	buffer++;
	memcpy((char *)&Rc_PPMEncoder1_power, buffer, 1);
	buffer++;
}

uint16_t Rc_Cmd_PPMEncoder2_Get (uint8_t *buffer) {
	uint16_t num = 0;
	buffer[num++] = Rc_PPMEncoder2_channels;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder2_shift, 2);
	num += 2;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder2_sync, 2);
	num += 2;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder2_center, 2);
	num += 2;
	memcpy((char *)buffer + num, (char *)&Rc_PPMEncoder2_range, 2);
	num += 2;
	buffer[num++] = Rc_PPMEncoder2_invert;
	buffer[num++] = Rc_PPMEncoder2_power;
	return num;
}

void Rc_Cmd_PPMEncoder2_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_PPMEncoder2_channels, buffer, 1);
	buffer++;
	memcpy((char *)&Rc_PPMEncoder2_shift, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder2_sync, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder2_center, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder2_range, buffer, 2);
	buffer += 2;
	memcpy((char *)&Rc_PPMEncoder2_invert, buffer, 1);
	buffer++;
	memcpy((char *)&Rc_PPMEncoder2_power, buffer, 1);
	buffer++;
}

uint16_t Rc_Cmd_PPMEncoder1_Mencoder_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer + num, &Rc_PPMEncoder1_Mencoder_max, PPM1_MENCODER_MAX);
	num += PPM1_MENCODER_MAX;
	memcpy((char *)buffer + num, &Rc_PPMEncoder1_Mencoder_channel, PPM1_MENCODER_MAX);
	num += PPM1_MENCODER_MAX;
	memcpy((char *)buffer + num, &Rc_PPMEncoder1_Mencoder_values, PPM1_MENCODER_MAX * PPM1_MENCODER_CHANNEL_MAX * 2);
	num += PPM1_MENCODER_MAX * PPM1_MENCODER_CHANNEL_MAX * 2;
	memcpy((char *)buffer + num, &Rc_PPMEncoder1_Mencoder_type, PPM1_MENCODER_MAX);
	num += PPM1_MENCODER_MAX;
	return num;
}

void Rc_Cmd_PPMEncoder1_Mencoder_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_PPMEncoder1_Mencoder_max, buffer, PPM1_MENCODER_MAX);
	buffer += PPM1_MENCODER_MAX;
	memcpy((char *)&Rc_PPMEncoder1_Mencoder_channel, buffer, PPM1_MENCODER_MAX);
	buffer += PPM1_MENCODER_MAX;
	memcpy((char *)&Rc_PPMEncoder1_Mencoder_values, buffer, PPM1_MENCODER_MAX * PPM1_MENCODER_CHANNEL_MAX * 2);
	buffer += PPM1_MENCODER_MAX * PPM1_MENCODER_CHANNEL_MAX * 2;
	memcpy((char *)&Rc_PPMEncoder1_Mencoder_type, buffer, PPM1_MENCODER_MAX);
	buffer += PPM1_MENCODER_MAX;
}



int8_t ControlSet(uint8_t control, int8_t value) {
	if (control == CONTROL_ON) {
		return value;
	}
	return value * ControlRead(control) / 100;
}

uint16_t Rc_Cmd_Control_Get (uint8_t *buffer) {
	uint16_t num = 0;
	for (num = 0; num < INPUT_ADC_MAX; num++) {
		Rc_Control_value[num] = ControlRead(num);
	}
	num = 0;
	memcpy((char *)buffer, &Rc_Control_value, CONTROL_MAX);
	num += CONTROL_MAX;
	memcpy((char *)buffer + num, &Rc_Control_reverse, CONTROL_MAX);
	num += CONTROL_MAX;
	return num;
}

void Rc_Cmd_Control_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Control_value, buffer, CONTROL_MAX);
	buffer += CONTROL_MAX;
	memcpy((char *)&Rc_Control_reverse, buffer, CONTROL_MAX);
	buffer += CONTROL_MAX;
}

uint16_t Rc_Cmd_LogicControl_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_LogicControl_target, LOGIC_MAX);
	num += LOGIC_MAX;
	memcpy((char *)buffer + num, &Rc_LogicControl_control, LOGIC_MAX);
	num += LOGIC_MAX;
	memcpy((char *)buffer + num, &Rc_LogicControl_test, LOGIC_MAX);
	num += LOGIC_MAX;
	memcpy((char *)buffer + num, &Rc_LogicControl_value, LOGIC_MAX);
	num += LOGIC_MAX;
	return num;
}

void Rc_Cmd_LogicControl_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_LogicControl_target, buffer, LOGIC_MAX);
	buffer += LOGIC_MAX;
	memcpy((char *)&Rc_LogicControl_control, buffer, LOGIC_MAX);
	buffer += LOGIC_MAX;
	memcpy((char *)&Rc_LogicControl_test, buffer, LOGIC_MAX);
	buffer += LOGIC_MAX;
	memcpy((char *)&Rc_LogicControl_value, buffer, LOGIC_MAX);
	buffer += LOGIC_MAX;
}

uint16_t Rc_Cmd_Output_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_Output_center_value, OUTPUT_MAX);
	num += OUTPUT_MAX;
	memcpy((char *)buffer + num, &Rc_Output_center_control, OUTPUT_MAX);
	num += OUTPUT_MAX;
	memcpy((char *)buffer + num, &Rc_Output_reverse, OUTPUT_MAX);
	num += OUTPUT_MAX;
	return num;
}

void Rc_Cmd_Output_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Output_center_value, buffer, OUTPUT_MAX);
	buffer += OUTPUT_MAX;
	memcpy((char *)&Rc_Output_center_control, buffer, OUTPUT_MAX);
	buffer += OUTPUT_MAX;
	memcpy((char *)&Rc_Output_reverse, buffer, OUTPUT_MAX);
	buffer += OUTPUT_MAX;
}

uint16_t Rc_Cmd_Expo_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_Expo_type, EXPO_MAX);
	num += EXPO_MAX;
	memcpy((char *)buffer + num, &Rc_Expo_channel, EXPO_MAX);
	num += EXPO_MAX;
	memcpy((char *)buffer + num, &Rc_Expo_switch, EXPO_MAX);
	num += EXPO_MAX;
	memcpy((char *)buffer + num, &Rc_Expo_gradient_value, EXPO_MAX);
	num += EXPO_MAX;
	memcpy((char *)buffer + num, &Rc_Expo_gradient_control, EXPO_MAX);
	num += EXPO_MAX;
	return num;
}

void Rc_Cmd_Expo_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Expo_type, buffer, EXPO_MAX);
	buffer += EXPO_MAX;
	memcpy((char *)&Rc_Expo_channel, buffer, EXPO_MAX);
	buffer += EXPO_MAX;
	memcpy((char *)&Rc_Expo_switch, buffer, EXPO_MAX);
	buffer += EXPO_MAX;
	memcpy((char *)&Rc_Expo_gradient_value, buffer, EXPO_MAX);
	buffer += EXPO_MAX;
	memcpy((char *)&Rc_Expo_gradient_control, buffer, EXPO_MAX);
	buffer += EXPO_MAX;
}

uint16_t Rc_Cmd_Dualrate_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_Dualrate_channel, DUALRATE_MAX);
	num += DUALRATE_MAX;
	memcpy((char *)buffer + num, &Rc_Dualrate_switch, DUALRATE_MAX);
	num += DUALRATE_MAX;
	memcpy((char *)buffer + num, &Rc_Dualrate_limit_value, DUALRATE_MAX);
	num += DUALRATE_MAX;
	memcpy((char *)buffer + num, &Rc_Dualrate_limit_control, DUALRATE_MAX);
	num += DUALRATE_MAX;
	return num;
}

void Rc_Cmd_Dualrate_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Dualrate_channel, buffer, DUALRATE_MAX);
	buffer += DUALRATE_MAX;
	memcpy((char *)&Rc_Dualrate_switch, buffer, DUALRATE_MAX);
	buffer += DUALRATE_MAX;
	memcpy((char *)&Rc_Dualrate_limit_value, buffer, DUALRATE_MAX);
	buffer += DUALRATE_MAX;
	memcpy((char *)&Rc_Dualrate_limit_control, buffer, DUALRATE_MAX);
	buffer += DUALRATE_MAX;
}

uint16_t Rc_Cmd_Limiter_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_Limiter_channel, LIMITER_MAX);
	num += LIMITER_MAX;
	memcpy((char *)buffer + num, &Rc_Limiter_switch, LIMITER_MAX);
	num += LIMITER_MAX;
	memcpy((char *)buffer + num, &Rc_Limiter_up_value, LIMITER_MAX);
	num += LIMITER_MAX;
	memcpy((char *)buffer + num, &Rc_Limiter_up_control, LIMITER_MAX);
	num += LIMITER_MAX;
	memcpy((char *)buffer + num, &Rc_Limiter_down_value, LIMITER_MAX);
	num += LIMITER_MAX;
	memcpy((char *)buffer + num, &Rc_Limiter_down_control, LIMITER_MAX);
	num += LIMITER_MAX;
	return num;
}

void Rc_Cmd_Limiter_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Limiter_channel, buffer, LIMITER_MAX);
	buffer += LIMITER_MAX;
	memcpy((char *)&Rc_Limiter_switch, buffer, LIMITER_MAX);
	buffer += LIMITER_MAX;
	memcpy((char *)&Rc_Limiter_up_value, buffer, LIMITER_MAX);
	buffer += LIMITER_MAX;
	memcpy((char *)&Rc_Limiter_up_control, buffer, LIMITER_MAX);
	buffer += LIMITER_MAX;
	memcpy((char *)&Rc_Limiter_down_value, buffer, LIMITER_MAX);
	buffer += LIMITER_MAX;
	memcpy((char *)&Rc_Limiter_down_control, buffer, LIMITER_MAX);
	buffer += LIMITER_MAX;
}

uint16_t Rc_Cmd_Atv_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_Atv_channel, ATV_MAX);
	num += ATV_MAX;
	memcpy((char *)buffer + num, &Rc_Atv_switch, ATV_MAX);
	num += ATV_MAX;
	memcpy((char *)buffer + num, &Rc_Atv_up_value, ATV_MAX);
	num += ATV_MAX;
	memcpy((char *)buffer + num, &Rc_Atv_up_control, ATV_MAX);
	num += ATV_MAX;
	memcpy((char *)buffer + num, &Rc_Atv_down_value, ATV_MAX);
	num += ATV_MAX;
	memcpy((char *)buffer + num, &Rc_Atv_down_control, ATV_MAX);
	num += ATV_MAX;
	return num;
}

void Rc_Cmd_Atv_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Atv_channel, buffer, ATV_MAX);
	buffer += ATV_MAX;
	memcpy((char *)&Rc_Atv_switch, buffer, ATV_MAX);
	buffer += ATV_MAX;
	memcpy((char *)&Rc_Atv_up_value, buffer, ATV_MAX);
	buffer += ATV_MAX;
	memcpy((char *)&Rc_Atv_up_control, buffer, ATV_MAX);
	buffer += ATV_MAX;
	memcpy((char *)&Rc_Atv_down_value, buffer, ATV_MAX);
	buffer += ATV_MAX;
	memcpy((char *)&Rc_Atv_down_control, buffer, ATV_MAX);
	buffer += ATV_MAX;
}

uint16_t Rc_Cmd_Motion_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_Motion_type, MOTION_MAX);
	num += MOTION_MAX;
	memcpy((char *)buffer + num, &Rc_Motion_channel, MOTION_MAX);
	num += MOTION_MAX;
	memcpy((char *)buffer + num, &Rc_Motion_softstart_speed, MOTION_MAX);
	num += MOTION_MAX;
	memcpy((char *)buffer + num, &Rc_Motion_switch, MOTION_MAX);
	num += MOTION_MAX;
	memcpy((char *)buffer + num, &Rc_Motion_up_value, MOTION_MAX);
	num += MOTION_MAX;
	memcpy((char *)buffer + num, &Rc_Motion_up_control, MOTION_MAX);
	num += MOTION_MAX;
	memcpy((char *)buffer + num, &Rc_Motion_down_value, MOTION_MAX);
	num += MOTION_MAX;
	memcpy((char *)buffer + num, &Rc_Motion_down_control, MOTION_MAX);
	num += MOTION_MAX;
	return num;
}

void Rc_Cmd_Motion_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Motion_type, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
	memcpy((char *)&Rc_Motion_channel, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
	memcpy((char *)&Rc_Motion_softstart_speed, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
	memcpy((char *)&Rc_Motion_switch, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
	memcpy((char *)&Rc_Motion_up_value, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
	memcpy((char *)&Rc_Motion_up_control, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
	memcpy((char *)&Rc_Motion_down_value, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
	memcpy((char *)&Rc_Motion_down_control, buffer, MOTION_MAX);
	buffer += MOTION_MAX;
}

uint16_t Rc_Cmd_Mixer_Get (uint8_t *buffer) {
	uint16_t num = 0;
	memcpy((char *)buffer, &Rc_Mixer_channel_in, MIXER_MAX);
	num += MIXER_MAX;
	memcpy((char *)buffer + num, &Rc_Mixer_channel_out, MIXER_MAX);
	num += MIXER_MAX;
	memcpy((char *)buffer + num, &Rc_Mixer_point, MIXER_MAX * (MIXER_POINTS_MAX * 2));
	num += MIXER_MAX * (MIXER_POINTS_MAX * 2);
	memcpy((char *)buffer + num, &Rc_Mixer_switch, MIXER_MAX);
	num += MIXER_MAX;
	memcpy((char *)buffer + num, &Rc_Mixer_percent_value, MIXER_MAX);
	num += MIXER_MAX;
	memcpy((char *)buffer + num, &Rc_Mixer_percent_control, MIXER_MAX);
	num += MIXER_MAX;
	return num;
}

void Rc_Cmd_Mixer_Set (uint8_t *buffer) {
	memcpy((char *)&Rc_Mixer_channel_in, buffer, MIXER_MAX);
	buffer += MIXER_MAX;
	memcpy((char *)&Rc_Mixer_channel_out, buffer, MIXER_MAX);
	buffer += MIXER_MAX;
	memcpy((char *)&Rc_Mixer_point, buffer, MIXER_MAX * (MIXER_POINTS_MAX * 2));
	buffer += MIXER_MAX * (MIXER_POINTS_MAX * 2);
	memcpy((char *)&Rc_Mixer_switch, buffer, MIXER_MAX);
	buffer += MIXER_MAX;
	memcpy((char *)&Rc_Mixer_percent_value, buffer, MIXER_MAX);
	buffer += MIXER_MAX;
	memcpy((char *)&Rc_Mixer_percent_control, buffer, MIXER_MAX);
	buffer += MIXER_MAX;
}

void Rc_Cmd_Init (void) {
	int16_t num;
	uint8_t point;
	uint8_t tmp_str[20];

	strcpy((char *)Rc_Model_name, "Defaults");
	ModelID = 0;
	Rc_Model_icon = 0;
	Servo_Testflag = 0;

	FeCmdLen[FE_CMD_CONTROL_VALUE_SET] = 2;
	FeCmdLen[FE_CMD_CONTROL_SET] = CONTROL_MAX * 2;
	FeCmdLen[FE_CMD_PPMENCODER1_MENCODER_SET] = PPM1_MENCODER_MAX * (3 + (PPM1_MENCODER_CHANNEL_MAX * 2));
	FeCmdLen[FE_CMD_PPMENCODER1_SET] = 11;
	FeCmdLen[FE_CMD_PPMENCODER2_SET] = 11;
	FeCmdLen[FE_CMD_OUTPUT_SET] = OUTPUT_MAX * 3;
	FeCmdLen[FE_CMD_EXPO_SET] = EXPO_MAX * 5;
	FeCmdLen[FE_CMD_LIMITER_SET] = LIMITER_MAX * 6;
	FeCmdLen[FE_CMD_ATV_SET] = ATV_MAX * 6;
	FeCmdLen[FE_CMD_DUALRATE_SET] = DUALRATE_MAX * 4;
	FeCmdLen[FE_CMD_MOTION_SET] = MOTION_MAX * 8;
	FeCmdLen[FE_CMD_MIXER_SET] = MIXER_MAX * (5 + (MIXER_POINTS_MAX * 2));

	for (num = 0; num < TIMER_MAX; num++) {
		SystemTimer_time[num] = 0;
		SystemTimer_stat[num] = 1;
		strcpy((char *)SystemTimer_name[num], "Timer- ");
		SystemTimer_name[num][6] = num + '0';
		SystemTimer_name[num][7] = 0;
			SystemTimer_control[num] = CONTROL_ON;
	}

	for (num = 0; num < 4; num++) {
		Rc_Calibrate_center[num] = 0;
		Rc_Calibrate_min[num] = -1024;
		Rc_Calibrate_max[num] = +1024;
	}
	for (num = 0; num < CONTROL_MAX; num++) {
		if (num == CONTROL_OFF) {
			strcpy((char *)Rc_Control_name[num], "Off");
		} else if (num == CONTROL_ON) {
			strcpy((char *)Rc_Control_name[num], "On");
		} else if (num < 20) {
			char tmp_str[] = "Geber- ";
			tmp_str[6] = 'A' + num;
			strcpy((char *)Rc_Control_name[num], (char *)tmp_str);
		} else if (num >= 20 && num < 50) {
			char tmp_str[] = "Geber  ";
			int2str((char *)tmp_str + 5, num - 19, 0);
			strcpy((char *)Rc_Control_name[num], (char *)tmp_str);
		} else {
			int2str((char *)tmp_str, num, 0);
			strcpy((char *)Rc_Control_name[num], (char *)tmp_str);
		}

		Rc_Control_value[num] = 0;
		Rc_Control_reverse[num] = 0;
		Rc_Control_buffer[num] = 0;
	}
	for (num = 0; num < LOGIC_MAX; num++) {
		Rc_LogicControl_target[num] = CONTROL_NONE;
		Rc_LogicControl_control[num] = CONTROL_NONE;
		Rc_LogicControl_test[num] = LOGIG_TEST_NONE;
		Rc_LogicControl_value[num] = 0;
	}
	for (num = 0; num < DUALRATE_MAX; num++) {
		Rc_Dualrate_channel[num] = 255;
		Rc_Dualrate_switch[num] = CONTROL_ON;
		Rc_Dualrate_limit_value[num] = 80;
		Rc_Dualrate_limit_control[num] = CONTROL_ON;
	}
	for (num = 0; num < EXPO_MAX; num++) {
		Rc_Expo_type[num] = 0;
		Rc_Expo_channel[num] = 255;
		Rc_Expo_switch[num] = CONTROL_ON;
		Rc_Expo_gradient_value[num] = 50;
		Rc_Expo_gradient_control[num] = CONTROL_ON;
	}
	for (num = 0; num < MIXER_MAX; num++) {
		Rc_Mixer_channel_in[num] = CONTROL_OFF;
		Rc_Mixer_channel_out[num] = 255;
		for (point = 0; point < MIXER_POINTS_MAX; point++) {
			Rc_Mixer_point[num][point][0] = point * 100 / (MIXER_POINTS_MAX - 1);
			Rc_Mixer_point[num][point][1] = point * 100 / (MIXER_POINTS_MAX - 1);
		}
		Rc_Mixer_switch[num] = CONTROL_ON;
		Rc_Mixer_percent_value[num] = 100;
		Rc_Mixer_percent_control[num] = CONTROL_ON;
	}
	for (num = 0; num < MOTION_MAX; num++) {
		Rc_Motion_buffer[num] = 0;
		Rc_Motion_softstart_speed[num] = 10;
		Rc_Motion_softstart_buffer[num] = 0;
		Rc_Motion_type[num] = 0;
		Rc_Motion_channel[num] = 255;
		Rc_Motion_switch[num] = CONTROL_ON;
		Rc_Motion_up_value[num] = 50;
		Rc_Motion_up_control[num] = CONTROL_ON;
		Rc_Motion_down_value[num] = 50;
		Rc_Motion_down_control[num] = CONTROL_ON;
	}
	for (num = 0; num < LIMITER_MAX; num++) {
		Rc_Limiter_channel[num] = 255;
		Rc_Limiter_switch[num] = CONTROL_ON;
		Rc_Limiter_up_value[num] = 100;
		Rc_Limiter_up_control[num] = CONTROL_ON;
		Rc_Limiter_down_value[num] = 100;
		Rc_Limiter_down_control[num] = CONTROL_ON;
	}
	for (num = 0; num < ATV_MAX; num++) {
		Rc_Atv_channel[num] = 255;
		Rc_Atv_switch[num] = CONTROL_ON;
		Rc_Atv_up_value[num] = 100;
		Rc_Atv_up_control[num] = CONTROL_ON;
		Rc_Atv_down_value[num] = 100;
		Rc_Atv_down_control[num] = CONTROL_ON;
	}
	for (num = 0; num < OUTPUT_MAX; num++) {
		if (num == 255) {
			strcpy((char *)Rc_Output_name[num], "No-Output");
		} else if (num < PPM1_CHANNELS_MAX) {
			char tmp_str[] = "PPM1-";
			int2str((char *)tmp_str + 5, num + 1, 0);
			strcpy((char *)Rc_Output_name[num], (char *)tmp_str);
		} else if (num < PPM1_CHANNELS_MAX + PPM2_CHANNELS_MAX) {
			char tmp_str[] = "PPM2-";
			int2str((char *)tmp_str + 5, num + 1, 0);
			strcpy((char *)Rc_Output_name[num], (char *)tmp_str);
		} else {
			int2str((char *)tmp_str, num + 1, 0);
			strcpy((char *)Rc_Output_name[num], (char *)tmp_str);
		}

		Rc_Output_value[num] = 0;
		Rc_Output_center_value[num] = 0;
		Rc_Output_center_control[num] = CONTROL_OFF;
		Rc_Output_reverse[num] = 0;
	}
	for (num = 0; num < PPM1_MENCODER_MAX; num++) {
		Rc_PPMEncoder1_Mencoder_type[num] = 0;
		Rc_PPMEncoder1_Mencoder_channel[num] = 255;
		Rc_PPMEncoder1_Mencoder_max[num] = PPM1_MENCODER_CHANNEL_MAX;
		for (point = 0; point < PPM1_MENCODER_CHANNEL_MAX; point++) {
			Rc_PPMEncoder1_Mencoder_values[num][point] = 0;
		}
	}

	// Default-Setup
	Rc_PPMEncoder1_channels = 8;
	Rc_PPMEncoder1_shift = 30 * 150; // 100/ms
	Rc_PPMEncoder1_center = 150 * 150; // 100/ms
	Rc_PPMEncoder1_range = 50 * 150; // (+-) 100/ms
	Rc_PPMEncoder1_sync = 120 * 500; // 12/ms
	Rc_PPMEncoder1_power = 0;
	Rc_PPMEncoder1_frame_sync = 0;

#ifdef RCOS_MAINBOARD_V1_0
	Rc_PPMEncoder1_invert = 0;
#endif
#ifdef RCOS_MAINBOARD_FC16
	Rc_PPMEncoder1_invert = 1;
#endif


	Rc_PPMEncoder2_channels = 8;
	Rc_PPMEncoder2_shift = 30 * 150; // 100/ms
	Rc_PPMEncoder2_center = 150 * 150; // 100/ms
	Rc_PPMEncoder2_range = 50 * 150; // (+-) 100/ms
	Rc_PPMEncoder2_sync = 110 * 500; // 11/ms
	Rc_PPMEncoder2_invert = 0;
	Rc_PPMEncoder2_power = 0;
	Rc_PPMEncoder2_frame_sync = 0;

/*
	// Multichannel-Test
	num = 0;
	Rc_PPMEncoder1_Mencoder_channel[num] = 4;
	Rc_PPMEncoder1_Mencoder_max[num] = 4;
	for (point = 0; point < PPM1_MENCODER_CHANNEL_MAX; point++) {
		Rc_PPMEncoder1_Mencoder_values[num][point] = 0;
	}
	num = 1;
	Rc_PPMEncoder1_Mencoder_channel[num] = 5;
	Rc_PPMEncoder1_Mencoder_max[num] = PPM1_MENCODER_CHANNEL_MAX;
	for (point = 0; point < PPM1_MENCODER_CHANNEL_MAX; point++) {
		Rc_PPMEncoder1_Mencoder_values[num][point] = 0;
	}
	// test-setup
	for (num = 0; num < CONTROL_MAX && num < MIXER_MAX && num < OUTPUT_MAX; num++) {
		Rc_Mixer_channel_in[num] = num;
		Rc_Mixer_channel_out[num] = num;
		Rc_Mixer_percent_value[num] = 100;
		Rc_Mixer_percent_control[num] = CONTROL_ON;
	}
	num = 0;
	Rc_Motion_softstart_speed[num] = 10;
	Rc_Motion_type[num] = MOTION_TYPE_INPUT_SET;
	Rc_Motion_channel[num] = 4;
	Rc_Motion_switch[num] = 255;
	Rc_Motion_up_value[num] = 50;
	Rc_Motion_up_control[num] = 255;
	Rc_Motion_down_value[num] = 50;
	Rc_Motion_down_control[num] = 255;
*/


}


