/*
Copyright (C) 2008-2009  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/

#include <stdint.h>
#include <string.h>
#include <rc.h>
#include <rc_shiftreg.h>
#include <rc_ppm-encoder.h>
#include <rc_adc.h>
#include <rc_control.h>
#include <spi_sw.h>
#include <sd.h>
#include <glcd.h>
#include <lcd.h>
#include <sound.h>
#include <rc_sd.h>


#ifdef SUPPORT_MMC

uint8_t load_name_mmc (char *name_str, uint8_t model_id) {
	uint8_t big_buffer[1030];
	unsigned long sector = (model_id + 1) * 20;

	// Read MMC-Header
	mmc_read_sector(0, big_buffer);
	big_buffer[20] = 0;
	if (strcmp((char *)big_buffer, "rCOS-mmc FS") == 0) {

		// Read Model-Name
		mmc_read_sector(sector++, big_buffer);
		big_buffer[100] = 0;
		if (strcmp((char *)big_buffer, "rCOS Setup-v0.3e") == 0) {
			strncpy((char *)name_str, (char *)big_buffer + 40, 33);
			name_str[32] = 0;
			return 0;
		}
	}
	name_str[0] = 0;
	return 1;
}

uint8_t load_setup_mmc (uint16_t model_id) {
	uint8_t num = 0;
	uint8_t big_buffer[2100];
	uint16_t sector = (model_id + 1) * 20;

	LCD_Clear();
	LCD_Font(2);
	LCD_Print_Center(0, "Loading Model", LCD_XOR, LCD_TRANS);
	BootMSG(10, "Check MMC..");

	// Read MMC-Header
	mmc_read_sector(0, big_buffer);
	big_buffer[20] = 0;

	if (strcmp((char *)big_buffer, "rCOS-mmc FS") == 0) {

		BootMSG(10, "Check MMC...Ok");

		if (model_id == 255) {
			model_id = big_buffer[30];
			sector = (model_id + 1) * 20;
		} else {
			// Write new Default-Model-ID
			big_buffer[30] = model_id;
			mmc_write_sector(0, big_buffer);
		}
		// Read-Stick-Calibration
		for (num = 0; num < 4; num++) {
			Rc_Calibrate_center[num] = (big_buffer[40 + num * 8]<<8);
			Rc_Calibrate_center[num] += big_buffer[40 + num * 8 + 1];
			Rc_Calibrate_min[num] = (big_buffer[40 + num * 8 + 2]<<8);
			Rc_Calibrate_min[num] += big_buffer[40 + num * 8 + 3];
			Rc_Calibrate_max[num] = (big_buffer[40 + num * 8 + 4]<<8);
			Rc_Calibrate_max[num] += big_buffer[40 + num * 8 + 5];
		}


		// Read Model-Data
		mmc_read_sector(sector++, big_buffer);
		big_buffer[100] = 0;

		if (strcmp((char *)big_buffer, "rCOS Setup-v0.3e") == 0) {

			BootMSG(50, "Load Model..");

			strncpy((char *)Rc_Model_name, (char *)big_buffer + 40, MODEL_NAME_MAX - 1);
			Rc_Model_name[MODEL_NAME_MAX - 1] = 0;

			ModelID = model_id;
			Rc_Model_icon = big_buffer[38];

			mmc_read_sector(sector++, big_buffer);
//			memcpy(&Rc_Control_name, big_buffer, CONTROL_MAX * NAME_MAX);
			memcpy(&Rc_Control_name, big_buffer, 30 * NAME_MAX);

			mmc_read_sector(sector++, big_buffer);
//			memcpy(&Rc_Output_name, big_buffer, OUTPUT_MAX * NAME_MAX);
			memcpy(&Rc_Output_name, big_buffer, 30 * NAME_MAX);

			mmc_read_sector(sector++, big_buffer);
			mmc_read_sector(sector++, big_buffer + 512);
			Rc_Cmd_Control_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_LogicControl_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			mmc_read_sector(sector++, big_buffer + 512);
			mmc_read_sector(sector++, big_buffer + 512 + 512);
			mmc_read_sector(sector++, big_buffer + 512 + 512 + 512);
			Rc_Cmd_Mixer_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_Output_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_Dualrate_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_Expo_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_Motion_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_Limiter_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_Atv_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_PPMEncoder1_Mencoder_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_PPMEncoder1_Set(big_buffer);

			mmc_read_sector(sector++, big_buffer);
			Rc_Cmd_PPMEncoder2_Set(big_buffer);

			BootMSG(100, "Load Model...done");

			return 0;
		}
	}

	BootMSG(50, "Check MMC...failed");
	LCD_Update();
	MotionTimer = 0; while (MotionTimer < 250) {}
	MotionTimer = 0; while (MotionTimer < 250) {}
	return 1;
}

uint8_t save_setup_mmc (uint16_t model_id) {
	uint8_t num = 0;
	uint8_t big_buffer[2100];
	uint16_t sector = (model_id + 1) * 20;

	LCD_Clear();
	LCD_Font(2);
	LCD_Print_Center(0, "Saving Model", LCD_XOR, LCD_TRANS);

	BootMSG(10, "Write Model..");


	// Write MMC-Header
	strcpy((char *)big_buffer, "rCOS-mmc FS");

	// Write Default-Model-ID
	big_buffer[30] = model_id;

	// Write-Stick-Calibration
	for (num = 0; num < 4; num++) {
		big_buffer[40 + num * 8] = ((Rc_Calibrate_center[num] & 0xFF00)>>8);
		big_buffer[40 + num * 8 + 1] = (Rc_Calibrate_center[num] & 0x00FF);
		big_buffer[40 + num * 8 + 2] = ((Rc_Calibrate_min[num] & 0xFF00)>>8);
		big_buffer[40 + num * 8 + 3] = (Rc_Calibrate_min[num] & 0x00FF);
		big_buffer[40 + num * 8 + 4] = ((Rc_Calibrate_max[num] & 0xFF00)>>8);
		big_buffer[40 + num * 8 + 5] = (Rc_Calibrate_max[num] & 0x00FF);
	}
	mmc_write_sector(0, big_buffer);
	ModelID = model_id;

	// Write Model-Data
	strcpy((char *)big_buffer, "rCOS Setup-v0.3e");
	Rc_Model_name[MODEL_NAME_MAX - 1] = 0;
	strncpy((char *)big_buffer + 40, (char *)Rc_Model_name, MODEL_NAME_MAX - 1);
	big_buffer[38] = Rc_Model_icon;
	mmc_write_sector(sector++, big_buffer);

//	memcpy(big_buffer, &Rc_Control_name, CONTROL_MAX * NAME_MAX);
	memcpy(big_buffer, &Rc_Control_name, 30 * NAME_MAX);
	mmc_write_sector(sector++, big_buffer);

//	memcpy(big_buffer, &Rc_Output_name, OUTPUT_MAX * NAME_MAX);
	memcpy(big_buffer, &Rc_Output_name, 30 * NAME_MAX);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_Control_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);
	mmc_write_sector(sector++, big_buffer + 512);

	Rc_Cmd_LogicControl_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_Mixer_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);
	mmc_write_sector(sector++, big_buffer + 512);
	mmc_write_sector(sector++, big_buffer + 512 + 512);
	mmc_write_sector(sector++, big_buffer + 512 + 512 + 512);

	Rc_Cmd_Output_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_Dualrate_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_Expo_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_Motion_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_Limiter_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_Atv_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_PPMEncoder1_Mencoder_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_PPMEncoder1_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	Rc_Cmd_PPMEncoder2_Get(big_buffer);
	mmc_write_sector(sector++, big_buffer);

	BootMSG(100, "Write Model...done");

	MotionTimer = 0; while (MotionTimer < 250) {}
	MotionTimer = 0; while (MotionTimer < 250) {}
	return 0;
}

void rc_mmc_init (void) {
	// init MMC
	BootMSG(100, "MMC-Init");
	LCD_Clear();
	LCD_Font(2);
	LCD_Print_Center(0, "MMC-Card", LCD_XOR, LCD_TRANS);

	BootMSG(10, "MMC-Init..");
	if (mmc_init() == 0) {
		BootMSG(10, "MMC-Init...Ok");
		sound(1000, 100);
		load_setup_mmc(255);
	} else {
		BootMSG(10, "MMC-Init...failed");
		sound(1500, 200);
		MotionTimer = 0; while (MotionTimer < 250) {}
		MotionTimer = 0; while (MotionTimer < 250) {}
		MotionTimer = 0; while (MotionTimer < 250) {}
		MotionTimer = 0; while (MotionTimer < 250) {}
	}
}

#endif

