/*
Copyright (C) 2008  Oliver Dippel <o.dippel@gmx.de>

This library is free software; you can redistribute it and/or
modify it under the terms of the GNU Lesser General Public
License as published by the Free Software Foundation; either
version 2.1 of the License, or (at your option) any later version.

This library is distributed in the hope that it will be useful,
but WITHOUT ANY WARRANTY; without even the implied warranty of
MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
Lesser General Public License for more details.

You should have received a copy of the GNU Lesser General Public
License along with this library; if not, write to the Free Software
Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

This program is free software; you can redistribute it and/or modify it
under the terms of the GNU General Public License as published by the
Free Software Foundation; either version 2 of the License, or (at
your option) any later version.

On Debian GNU/Linux systems, the complete text of the GNU General
Public License can be found in `/usr/share/common-licenses/GPL'.
*/


#include "stm32f10x_conf.h"

#include "stm32f10x_lib.h"

#include "rc.h"

#include "rc_ppm-encoder.h"


/*
-----------------------
MCU               RFM01
-----------------------
SCK               SCK
MISO              SDO
MOSI              SDI
SS                nSEL
INT0              nIRQ
-----------------------
*/




#define PORT_IRQ GPIOC
#define RFM01_IRQ GPIO_Pin_6

#define PORT_SEL GPIOB
#define RFM01_SEL GPIO_Pin_12

#define PORT_SCK GPIOB
#define RFM01_SCK GPIO_Pin_13

#define PORT_SDO GPIOB
#define RFM01_SDO GPIO_Pin_14

#define PORT_SDI GPIOB
#define RFM01_SDI GPIO_Pin_15


#define IRQ_HI() (GPIO_ReadInputDataBit(PORT_IRQ, RFM01_IRQ) != 0)
#define IRQ_LO() (GPIO_ReadInputDataBit(PORT_IRQ, RFM01_IRQ) == 0)

#define SDO_HI() (GPIO_ReadInputDataBit(PORT_SDO, RFM01_SDO) != 0)

#define HI_SEL() GPIO_SetBits(PORT_SEL, RFM01_SEL)
#define LOW_SEL() GPIO_ResetBits(PORT_SEL, RFM01_SEL)

#define HI_SDI() GPIO_SetBits(PORT_SDI, RFM01_SDI)
#define LOW_SDI() GPIO_ResetBits(PORT_SDI, RFM01_SDI)

#define HI_SCK() GPIO_SetBits(PORT_SCK, RFM01_SCK);
#define LOW_SCK() GPIO_ResetBits(PORT_SCK, RFM01_SCK);


#define RFM01FREQ(freq) ((freq-430.0)/0.0025)


typedef struct {
	uint8_t receiver_id;	// Empfnger-ID
	uint16_t value[6];	// Analog-Wert
	uint16_t gps_lat[2];	// GPS-Lat
	uint8_t gps_lat_ns;	// GPS-Lat N/S
	uint16_t gps_long[2];	// GPS-Long
	uint8_t gps_long_ew;	// GPS-Long E/W
	uint16_t gps_alti;	// GPS-Hhe
	uint16_t gps_speed;	// GPS-Speed
	uint16_t checksum;
} SendFrame;

SendFrame frame_data;

uint8_t RF_RXBUF[30];


uint16_t RFM01_WRT_CMD (uint16_t aCmd) {
	uint8_t i = 0;
	uint16_t temp = 0;
	LOW_SCK();
	LOW_SEL();
	MotionTimer = 0; while (MotionTimer < 7) {}
	for(i=0;i<16;i++){
		temp<<=1;
		if(SDO_HI()){
			temp |= 0x0001;
		}
		LOW_SCK();
		MotionTimer = 0; while (MotionTimer < 7) {}
		if(aCmd&0x8000){
			HI_SDI();
		}else{
			LOW_SDI();
		}
		HI_SCK();
		MotionTimer = 0; while (MotionTimer < 7) {}
		aCmd<<=1;

	};
	LOW_SCK();
	HI_SEL();

	if (temp == 0) {
		USART2_SEND_STR("### 0 ###\r\n");
	} else {
		USART2_SEND_STR("### 1 ###\r\n");
	}
	USART2_TX_START_OK();
	MotionTimer = 0; while (MotionTimer < 150) {}



	MotionTimer = 0; while (MotionTimer < 7) {}
	return(temp);
}

void RFM01_PORT_INIT (void) {
	GPIO_InitTypeDef GPIO_InitStructure;

	GPIO_InitStructure.GPIO_Pin = RFM01_IRQ;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(PORT_IRQ, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = RFM01_SDO;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_IPU;
	GPIO_Init(PORT_SDO, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = PORT_SDI;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(PORT_SDI, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = PORT_SCK;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(PORT_SCK, &GPIO_InitStructure);

	GPIO_InitStructure.GPIO_Pin = PORT_SEL;
	GPIO_InitStructure.GPIO_Speed = GPIO_Speed_50MHz;
	GPIO_InitStructure.GPIO_Mode = GPIO_Mode_Out_PP;
	GPIO_Init(PORT_SEL, &GPIO_InitStructure);

	HI_SEL();
	HI_SDI();
	LOW_SCK();
}

void RFM01_PORT_SETUP (void) {
	RFM01_WRT_CMD(0x0000);
	RFM01_WRT_CMD(0x898A); //134kHz
//	RFM01_WRT_CMD(RFM01FREQ(433.92)); //434MHz (Erlaubt(DE): 433.05MHz -> 434.790MHz)
	RFM01_WRT_CMD(0xA640); //434MHz (Erlaubt(DE): 433.05MHz -> 434.790MHz)
	RFM01_WRT_CMD(0xC847); //4.8kbps
	RFM01_WRT_CMD(0xC69B); //AFC setting
	RFM01_WRT_CMD(0xC42A); //Clock recovery manual control,Digital filter,DQD=4
	RFM01_WRT_CMD(0xC240); //output 1.66MHz
	RFM01_WRT_CMD(0xC080);
	RFM01_WRT_CMD(0xCE88); //use FIFO
	RFM01_WRT_CMD(0xCE8B);
	RFM01_WRT_CMD(0xC081); //OPEN RX
}


uint8_t RFM01_RDFIFO (void) {
	uint8_t i = 0;
	uint8_t Result = 0;
	LOW_SCK();
	LOW_SDI();
	LOW_SEL();
	MotionTimer = 0; while (MotionTimer < 7) {}
	for(i = 0; i < 16; i++){ //skip status bits
		HI_SCK();
		MotionTimer = 0; while (MotionTimer < 7) {}
		LOW_SCK();
		MotionTimer = 0; while (MotionTimer < 7) {}
	}
	for(i = 0; i < 8; i++){ //read fifo data byte
		Result<<=1;
		if(SDO_HI()){
			Result |=  1;
		}
		HI_SCK();
		MotionTimer = 0; while (MotionTimer < 7) {}
		LOW_SCK();
		MotionTimer = 0; while (MotionTimer < 7) {}
	};
	HI_SEL();
	MotionTimer = 0; while (MotionTimer < 7) {}
	return(Result);
}


void Telemetrie_Init (void) {
	RFM01_PORT_INIT();
	RFM01_PORT_SETUP();

}


extern void USART2_SEND (char *data, uint16_t count);
extern void USART2_SEND_STR (char *data);


void Telemetrie_Check (void) {
	static uint8_t tele_n = 0;
	uint8_t j = 0;
	uint8_t ChkSum = 0;
	while (IRQ_LO()) { //polling the nIRQ data
		RF_RXBUF[tele_n++] = RFM01_RDFIFO(); //read FIFO data

//		USART2_SEND(RF_RXBUF, tele_n - 1);
		USART2_SEND_STR("#######\r\n");
		USART2_TX_START_OK();
		MotionTimer = 0; while (MotionTimer < 150) {}

		if (tele_n == sizeof(SendFrame)){
			RFM01_WRT_CMD(0xCE88); //reset FIFO for next frame recognition
			RFM01_WRT_CMD(0xCE8B);
			ChkSum = 0;
			tele_n = 0;
			if (RF_RXBUF[0] == 99) {
				for (j = 0; j < sizeof(SendFrame) - 2; j++){
					ChkSum += RF_RXBUF[j]; //calculate checksum
				}
				if (ChkSum == RF_RXBUF[sizeof(SendFrame) - 2]){ //frame check
					memcpy(&frame_data, &RF_RXBUF, sizeof(SendFrame));
				}
			}
		}
	}
}



